#!/usr/bin/env python3

"""
#   Copyright (C) 2004 Midori (midori -- a-t -- paipai dot net)
#   Copyright (C) 2006-2018 Ludovic Rousseau (ludovic.rousseau@free.fr)
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA.
"""

import os

# pylint: disable=duplicate-code
# pylint: disable=wildcard-import
# pylint: disable=unused-wildcard-import
from PyKCS11.LowLevel import *

a = CPKCS11Lib()
info = CK_INFO()
slotInfo = CK_SLOT_INFO()
lib = os.getenv("PYKCS11LIB")
if lib is None:
    raise ValueError("Define PYKCS11LIB")
session = CK_SESSION_HANDLE()
sessionInfo = CK_SESSION_INFO()
tokenInfo = CK_TOKEN_INFO()
slotList = ckulonglist()
pin = ckbytelist(b"1234")

print("Load of " + lib + ": " + str(a.Load(lib)))
print("C_GetInfo:", hex(a.C_GetInfo(info)))
print("Library manufacturerID:", info.GetManufacturerID())
del info

print("C_GetSlotList(NULL): " + hex(a.C_GetSlotList(0, slotList)))
print("\tAvailable Slots: " + str(len(slotList)))

for index, slot in enumerate(slotList):
    print("\tC_SlotInfo(): " + hex(a.C_GetSlotInfo(slot, slotInfo)))
    print(
        "\t\tSlot N."
        + str(index)
        + ": ID="
        + str(slot)
        + ", name='"
        + slotInfo.GetSlotDescription()
        + "'"
    )
    print(
        "\tC_OpenSession(): "
        + hex(a.C_OpenSession(slot, CKF_SERIAL_SESSION | CKF_RW_SESSION, session))
    )
    print("\t\tSession:" + str(session))
    print("\tC_GetSessionInfo(): " + hex(a.C_GetSessionInfo(session, sessionInfo)))
    print(
        "\t\tSessionInfo: state="
        + hex(sessionInfo.state)
        + ", flags="
        + hex(sessionInfo.flags)
    )

    print("\tC_GetTokenInfo(): " + hex(a.C_GetTokenInfo(slot, tokenInfo)))
    print(
        "\t\tTokenInfo: Label="
        + tokenInfo.GetLabel()
        + ", ManufacturerID="
        + tokenInfo.GetManufacturerID()
    )
    print(
        "\t\tTokenInfo: flags="
        + hex(tokenInfo.flags)
        + ", Model="
        + tokenInfo.GetModel()
    )

    print("\tC_Login(): " + hex(a.C_Login(session, CKU_USER, pin)))
    print("\tC_Logout(): " + hex(a.C_Logout(session)))
    print("\tC_CloseSession(): " + hex(a.C_CloseSession(session)))

print(
    "C_OpenSession(): " + hex(a.C_OpenSession(slotList[0], CKF_SERIAL_SESSION, session))
)
print("C_Login(): " + hex(a.C_Login(session, CKU_USER, pin)))

SearchResult = ckulonglist(10)
SearchTemplate = ckattrlist(0)
# SearchTemplate[0].SetNum(CKA_CLASS, CKO_CERTIFICATE)
# SearchTemplate[1].SetBool(CKA_TOKEN, True)

print("C_FindObjectsInit: " + hex(a.C_FindObjectsInit(session, SearchTemplate)))
print("C_FindObjects: " + hex(a.C_FindObjects(session, SearchResult)))
print("C_FindObjectsFinal: " + hex(a.C_FindObjectsFinal(session)))

attributes = [
    ["CKA_CLASS", CKA_CLASS],
    ["CKA_TOKEN", CKA_TOKEN],
    ["CKA_PRIVATE", CKA_PRIVATE],
    ["CKA_LABEL", CKA_LABEL],
    ["CKA_APPLICATION", CKA_APPLICATION],
    ["CKA_VALUE", CKA_VALUE],
    ["CKA_CERTIFICATE_TYPE", CKA_CERTIFICATE_TYPE],
    ["CKA_ISSUER", CKA_ISSUER],
    ["CKA_SERIAL_NUMBER", CKA_SERIAL_NUMBER],
    ["CKA_KEY_TYPE", CKA_KEY_TYPE],
    ["CKA_SUBJECT", CKA_SUBJECT],
    ["CKA_ID", CKA_ID],
    ["CKA_SENSITIVE", CKA_SENSITIVE],
    ["CKA_ENCRYPT", CKA_ENCRYPT],
    ["CKA_DECRYPT", CKA_DECRYPT],
    ["CKA_WRAP", CKA_WRAP],
    ["CKA_UNWRAP", CKA_UNWRAP],
    ["CKA_SIGN", CKA_SIGN],
    ["CKA_SIGN_RECOVER", CKA_SIGN_RECOVER],
    ["CKA_VERIFY", CKA_VERIFY],
    ["CKA_VERIFY_RECOVER", CKA_VERIFY_RECOVER],
    ["CKA_DERIVE", CKA_DERIVE],
    ["CKA_START_DATE", CKA_START_DATE],
    ["CKA_END_DATE", CKA_END_DATE],
    ["CKA_MODULUS", CKA_MODULUS],
    ["CKA_MODULUS_BITS", CKA_MODULUS_BITS],
    ["CKA_PUBLIC_EXPONENT", CKA_PUBLIC_EXPONENT],
    ["CKA_PRIVATE_EXPONENT", CKA_PRIVATE_EXPONENT],
    ["CKA_PRIME_1", CKA_PRIME_1],
    ["CKA_PRIME_2", CKA_PRIME_2],
    ["CKA_EXPONENT_1", CKA_EXPONENT_1],
    ["CKA_EXPONENT_2", CKA_EXPONENT_2],
    ["CKA_COEFFICIENT", CKA_COEFFICIENT],
    ["CKA_PRIME", CKA_PRIME],
    ["CKA_SUBPRIME", CKA_SUBPRIME],
    ["CKA_BASE", CKA_BASE],
    ["CKA_VALUE_BITS", CKA_VALUE_BITS],
    ["CKA_VALUE_LEN", CKA_VALUE_LEN],
    ["CKA_EXTRACTABLE", CKA_EXTRACTABLE],
    ["CKA_LOCAL", CKA_LOCAL],
    ["CKA_NEVER_EXTRACTABLE", CKA_NEVER_EXTRACTABLE],
    ["CKA_ALWAYS_SENSITIVE", CKA_ALWAYS_SENSITIVE],
    ["CKA_MODIFIABLE", CKA_MODIFIABLE],
    ["CKA_ECDSA_PARAMS", CKA_ECDSA_PARAMS],
    ["CKA_EC_POINT", CKA_EC_POINT],
]

for x in SearchResult:
    print("object: " + hex(x.value()))
    valTemplate = ckattrlist(1)
    for attr in attributes:
        valTemplate[0].Reset()
        valTemplate[0].SetType(attr[1])
        # first call to get the attribute size and reserve the memory
        a.C_GetAttributeValue(session, x, valTemplate)
        # second call to get the attribute value
        rv = a.C_GetAttributeValue(session, x, valTemplate)
        if rv == CKR_OK:
            print("\t" + attr[0] + ": ", end=" ")
            if valTemplate[0].IsNum():
                print(valTemplate[0].GetNum())
            if valTemplate[0].IsBool():
                print(valTemplate[0].GetBool())
            if valTemplate[0].IsString():
                print(valTemplate[0].GetString())
            if valTemplate[0].IsBin():
                print("(" + str(valTemplate[0].GetLen()) + " bytes)", end=" ")
                print(list(map(hex, valTemplate[0].GetBin())))

print("C_Logout(): " + hex(a.C_Logout(session)))
print("C_CloseSession(): " + hex(a.C_CloseSession(session)))
print("C_Finalize(): " + hex(a.C_Finalize()))
print(a.Unload())



#!/usr/bin/env python3

"""
#   Copyright (C) 2019 Atte Pellikka <atte.pellikka@gmail.com>
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA.
"""

from asn1crypto.keys import ECDomainParameters, NamedCurve

import PyKCS11

pkcs11 = PyKCS11.PyKCS11Lib()
pkcs11.load()

slot = pkcs11.getSlotList(tokenPresent=True)[0]

session = pkcs11.openSession(slot, PyKCS11.CKF_SERIAL_SESSION | PyKCS11.CKF_RW_SESSION)
session.login("1234")

key_id = (0x22,)
label = "test"

# Select the curve to be used for the keys
curve = "secp256r1"

# Setup the domain parameters, unicode conversion needed for the curve string
domain_params = ECDomainParameters(name="named", value=NamedCurve(curve))
ec_params = domain_params.dump()

ec_public_tmpl = [
    (PyKCS11.CKA_CLASS, PyKCS11.CKO_PUBLIC_KEY),
    (PyKCS11.CKA_PRIVATE, PyKCS11.CK_FALSE),
    (PyKCS11.CKA_TOKEN, PyKCS11.CK_TRUE),
    (PyKCS11.CKA_ENCRYPT, PyKCS11.CK_TRUE),
    (PyKCS11.CKA_VERIFY, PyKCS11.CK_TRUE),
    (PyKCS11.CKA_WRAP, PyKCS11.CK_TRUE),
    (PyKCS11.CKA_KEY_TYPE, PyKCS11.CKK_ECDSA),
    (PyKCS11.CKA_EC_PARAMS, ec_params),
    (PyKCS11.CKA_LABEL, label),
    (PyKCS11.CKA_ID, key_id),
]

ec_priv_tmpl = [
    (PyKCS11.CKA_CLASS, PyKCS11.CKO_PRIVATE_KEY),
    (PyKCS11.CKA_KEY_TYPE, PyKCS11.CKK_ECDSA),
    (PyKCS11.CKA_TOKEN, PyKCS11.CK_TRUE),
    (PyKCS11.CKA_SENSITIVE, PyKCS11.CK_TRUE),
    (PyKCS11.CKA_DECRYPT, PyKCS11.CK_TRUE),
    (PyKCS11.CKA_SIGN, PyKCS11.CK_TRUE),
    (PyKCS11.CKA_UNWRAP, PyKCS11.CK_TRUE),
    (PyKCS11.CKA_LABEL, label),
    (PyKCS11.CKA_ID, key_id),
]

(pub_key, priv_key) = session.generateKeyPair(
    ec_public_tmpl, ec_priv_tmpl, mecha=PyKCS11.MechanismECGENERATEKEYPAIR
)

session.logout()
session.closeSession()



# pylint: disable=missing-module-docstring
# pylint: disable=missing-class-docstring
# pylint: disable=missing-function-docstring

import unittest

from asn1crypto.keys import ECDomainParameters, NamedCurve

from PyKCS11 import PyKCS11


class TestUtil(unittest.TestCase):
    def setUp(self):
        self.pkcs11 = PyKCS11.PyKCS11Lib()
        self.pkcs11.load()

        # get SoftHSM major version
        info = self.pkcs11.getInfo()
        self.SoftHSMversion = info.libraryVersion
        self.manufacturer = info.manufacturerID

        self.slot = self.pkcs11.getSlotList(tokenPresent=True)[0]
        self.session = self.pkcs11.openSession(
            self.slot, PyKCS11.CKF_SERIAL_SESSION | PyKCS11.CKF_RW_SESSION
        )
        self.session.login("1234")

        # common templates used in derive test cases
        self.aesKeyTemplate = [
            (PyKCS11.CKA_CLASS, PyKCS11.CKO_SECRET_KEY),
            (PyKCS11.CKA_KEY_TYPE, PyKCS11.CKK_AES),
            (PyKCS11.CKA_VALUE_LEN, 32),
            (PyKCS11.CKA_TOKEN, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_SENSITIVE, PyKCS11.CK_FALSE),
            (PyKCS11.CKA_EXTRACTABLE, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_PRIVATE, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_LABEL, "DeriveTestBaseAes256Key"),
            (PyKCS11.CKA_DERIVE, PyKCS11.CK_TRUE),
        ]

        self.genericKeyTemplate = [
            (PyKCS11.CKA_CLASS, PyKCS11.CKO_SECRET_KEY),
            (PyKCS11.CKA_KEY_TYPE, PyKCS11.CKK_GENERIC_SECRET),
            (PyKCS11.CKA_TOKEN, PyKCS11.CK_FALSE),
            (PyKCS11.CKA_SENSITIVE, PyKCS11.CK_FALSE),
            (PyKCS11.CKA_EXTRACTABLE, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_PRIVATE, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_LABEL, "DeriveTestGenericKey"),
        ]

        # generate a common symmetric base key for tests
        keyID = (0x01,)
        baseKeyTemplate = self.aesKeyTemplate + [(PyKCS11.CKA_ID, keyID)]
        mechanism = PyKCS11.Mechanism(PyKCS11.CKM_AES_KEY_GEN, None)
        self.baseKey = self.session.generateKey(baseKeyTemplate, mechanism)
        self.assertIsNotNone(self.baseKey)

        # Select the curve to be used for the keys
        curve = "secp256r1"

        # Setup the domain parameters, unicode conversion needed
        # for the curve string
        domain_params = ECDomainParameters(name="named", value=NamedCurve(curve))
        self.ecParams = domain_params.dump()

        keyID = (0x01,)
        baseKeyPubTemplate = [
            (PyKCS11.CKA_CLASS, PyKCS11.CKO_PUBLIC_KEY),
            (PyKCS11.CKA_PRIVATE, PyKCS11.CK_FALSE),
            (PyKCS11.CKA_TOKEN, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_ENCRYPT, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_VERIFY, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_WRAP, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_KEY_TYPE, PyKCS11.CKK_ECDSA),
            (PyKCS11.CKA_EC_PARAMS, self.ecParams),
            (PyKCS11.CKA_LABEL, "TestBaseKeyP256"),
            (PyKCS11.CKA_ID, keyID),
        ]
        baseKeyPvtTemplate = [
            (PyKCS11.CKA_CLASS, PyKCS11.CKO_PRIVATE_KEY),
            (PyKCS11.CKA_KEY_TYPE, PyKCS11.CKK_ECDSA),
            (PyKCS11.CKA_TOKEN, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_SENSITIVE, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_PRIVATE, PyKCS11.CK_FALSE),
            (PyKCS11.CKA_DECRYPT, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_SIGN, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_UNWRAP, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_LABEL, "TestBaseKeyP256"),
            (PyKCS11.CKA_ID, keyID),
            (PyKCS11.CKA_DERIVE, PyKCS11.CK_TRUE),
        ]
        mechanism = PyKCS11.Mechanism(PyKCS11.CKM_EC_KEY_PAIR_GEN, None)
        self.baseEcPubKey, self.baseEcPvtKey = self.session.generateKeyPair(
            baseKeyPubTemplate, baseKeyPvtTemplate, mechanism
        )
        self.assertIsNotNone(self.baseEcPubKey)
        self.assertIsNotNone(self.baseEcPvtKey)

    def tearDown(self):
        self.session.destroyObject(self.baseEcPubKey)
        self.session.destroyObject(self.baseEcPvtKey)

        self.session.logout()
        self.pkcs11.closeAllSessions(self.slot)
        self.pkcs11.unload()
        del self.pkcs11

    def getCkaValue(self, key):
        return list(self.session.getAttributeValue(key, [PyKCS11.CKA_VALUE])[0])

    def test_deriveKey_ECDH1_DERIVE(self):
        if self.SoftHSMversion[0] < 2:
            self.skipTest("generateKeyPair() only supported by SoftHSM >= 2")

        keyID = (0x11,)
        pubTemplate = [
            (PyKCS11.CKA_CLASS, PyKCS11.CKO_PUBLIC_KEY),
            (PyKCS11.CKA_PRIVATE, PyKCS11.CK_FALSE),
            (PyKCS11.CKA_TOKEN, PyKCS11.CK_FALSE),
            (PyKCS11.CKA_ENCRYPT, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_VERIFY, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_WRAP, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_KEY_TYPE, PyKCS11.CKK_ECDSA),
            (PyKCS11.CKA_EC_PARAMS, self.ecParams),
            (PyKCS11.CKA_LABEL, "testKeyP256"),
            (PyKCS11.CKA_ID, keyID),
        ]
        pvtTemplate = [
            (PyKCS11.CKA_CLASS, PyKCS11.CKO_PRIVATE_KEY),
            (PyKCS11.CKA_KEY_TYPE, PyKCS11.CKK_ECDSA),
            (PyKCS11.CKA_TOKEN, PyKCS11.CK_FALSE),
            (PyKCS11.CKA_SENSITIVE, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_PRIVATE, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_DECRYPT, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_SIGN, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_UNWRAP, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_LABEL, "testKeyP256"),
            (PyKCS11.CKA_ID, keyID),
            (PyKCS11.CKA_DERIVE, PyKCS11.CK_TRUE),
        ]
        mechanism = PyKCS11.Mechanism(PyKCS11.CKM_EC_KEY_PAIR_GEN, None)
        pubKey, pvtKey = self.session.generateKeyPair(
            pubTemplate, pvtTemplate, mechanism
        )
        self.assertIsNotNone(pubKey)
        self.assertIsNotNone(pvtKey)

        keyID = (0x22,)
        derivedAESKeyTemplate = [
            (PyKCS11.CKA_CLASS, PyKCS11.CKO_SECRET_KEY),
            (PyKCS11.CKA_KEY_TYPE, PyKCS11.CKK_AES),
            (PyKCS11.CKA_TOKEN, PyKCS11.CK_FALSE),
            (PyKCS11.CKA_SENSITIVE, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_PRIVATE, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_ENCRYPT, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_DECRYPT, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_SIGN, PyKCS11.CK_FALSE),
            (PyKCS11.CKA_EXTRACTABLE, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_VERIFY, PyKCS11.CK_FALSE),
            (PyKCS11.CKA_VALUE_LEN, 24),
            (PyKCS11.CKA_LABEL, "derivedAESKey"),
            (PyKCS11.CKA_ID, keyID),
        ]

        # derive key 1 : self.basePvtKey + pubKey
        attrs = self.session.getAttributeValue(pubKey, [PyKCS11.CKA_EC_POINT], True)
        mechanism = PyKCS11.ECDH1_DERIVE_Mechanism(bytes(attrs[0]))
        derivedKey = self.session.deriveKey(
            self.baseEcPvtKey, derivedAESKeyTemplate, mechanism
        )
        self.assertIsNotNone(derivedKey)

        # derive key 2 : pvtKey + self.basePubKey
        attrs = self.session.getAttributeValue(
            self.baseEcPubKey, [PyKCS11.CKA_EC_POINT], True
        )
        mechanism = PyKCS11.ECDH1_DERIVE_Mechanism(bytes(attrs[0]))
        derivedKey2 = self.session.deriveKey(pvtKey, derivedAESKeyTemplate, mechanism)
        self.assertIsNotNone(derivedKey2)

        DataIn = b"Sample data to test ecdh1 derive"
        mechanism = PyKCS11.Mechanism(PyKCS11.CKM_AES_CBC, "1234567812345678")
        DataOut = self.session.encrypt(derivedKey, DataIn, mechanism)
        DataCheck = self.session.decrypt(derivedKey2, DataOut, mechanism)

        # match check values
        self.assertSequenceEqual(DataIn, DataCheck)

        # cleanup
        self.session.destroyObject(derivedKey)
        self.session.destroyObject(derivedKey2)
        self.session.destroyObject(pubKey)
        self.session.destroyObject(pvtKey)

    def test_deriveKey_CKM_CONCATENATE_BASE_AND_KEY(self):
        # This mechanism is not supported in the current release of
        # SoftHSM (2.6.1), however available in develop branch,
        # see https://github.com/opendnssec/SoftHSMv2/commit/fa595c07a185656382c18ea2a6a12cad825d48b4
        if self.SoftHSMversion <= (2, 6):
            self.skipTest(
                "CKM_CONCATENATE_BASE_AND_KEY is not supported by SoftHSM <= 2.6"
            )

        # generate a key to concatenate with
        keyID = (0x11,)
        concatenateKeyTemplate = self.aesKeyTemplate + [(PyKCS11.CKA_ID, keyID)]
        key_gen_mechanism = PyKCS11.Mechanism(PyKCS11.CKM_AES_KEY_GEN, None)
        concKey = self.session.generateKey(concatenateKeyTemplate, key_gen_mechanism)
        self.assertIsNotNone(concKey)

        # concatenate two keys
        keyID = (0x22,)
        derivedKeyTemplate = self.genericKeyTemplate + [
            (PyKCS11.CKA_VALUE_LEN, 64),
            (PyKCS11.CKA_ID, keyID),
        ]
        mechanism = PyKCS11.CONCATENATE_BASE_AND_KEY_Mechanism(concKey)
        derivedKey = self.session.deriveKey(self.baseKey, derivedKeyTemplate, mechanism)
        self.assertIsNotNone(derivedKey)

        # check derived key's value
        baseKeyValue = self.getCkaValue(self.baseKey)
        concKeyValue = self.getCkaValue(concKey)
        derivedKeyValue = self.getCkaValue(derivedKey)

        # match: check values
        self.assertSequenceEqual(baseKeyValue + concKeyValue, derivedKeyValue)

        # check that mechanism shares ownership of temporary key
        mechanism = PyKCS11.CONCATENATE_BASE_AND_KEY_Mechanism(
            self.session.generateKey(concatenateKeyTemplate, key_gen_mechanism)
        )
        derivedKey = self.session.deriveKey(self.baseKey, derivedKeyTemplate, mechanism)
        self.assertIsNotNone(derivedKey)

        # cleanup
        self.session.destroyObject(derivedKey)
        self.session.destroyObject(concKey)

    def test_deriveKey_CKM_CONCATENATE_BASE_AND_DATA(self):
        # This mechanism is not supported in the current release of
        # SoftHSM (2.6.1), however available in develop branch,
        # see https://github.com/opendnssec/SoftHSMv2/commit/dba00d73e1b69f65b68397d235e7f73bbf59ab6a
        if self.SoftHSMversion <= (2, 6):
            self.skipTest(
                "CKM_CONCATENATE_BASE_AND_DATA is not supported by SoftHSM <= 2.6"
            )

        # generate data to concatenate with
        concData = list(self.session.generateRandom(32))

        # concatenate key with data
        keyID = (0x22,)
        derivedKeyTemplate = self.genericKeyTemplate + [
            (PyKCS11.CKA_VALUE_LEN, 64),
            (PyKCS11.CKA_ID, keyID),
        ]
        mechanism = PyKCS11.CONCATENATE_BASE_AND_DATA_Mechanism(concData)
        derivedKey = self.session.deriveKey(self.baseKey, derivedKeyTemplate, mechanism)
        self.assertIsNotNone(derivedKey)

        # check derived key's value
        baseKeyValue = self.getCkaValue(self.baseKey)
        derivedKeyValue = self.getCkaValue(derivedKey)

        # match: check values
        self.assertSequenceEqual(baseKeyValue + concData, derivedKeyValue)

        # cleanup
        self.session.destroyObject(derivedKey)

    def test_deriveKey_CKM_CONCATENATE_DATA_AND_BASE(self):
        # This mechanism is not supported in the current release of
        # SoftHSM (2.6.1), however available in develop branch,
        # see https://github.com/opendnssec/SoftHSMv2/commit/fae0d9f769ac30d25f563c5fc6c417e9199e4403
        if self.SoftHSMversion <= (2, 6):
            self.skipTest(
                "CKM_CONCATENATE_DATA_AND_BASE is not supported by SoftHSM <= 2.6"
            )

        # generate data to concatenate with
        concData = list(self.session.generateRandom(32))

        # concatenate data with key
        keyID = (0x22,)
        derivedKeyTemplate = self.genericKeyTemplate + [
            (PyKCS11.CKA_VALUE_LEN, 64),
            (PyKCS11.CKA_ID, keyID),
        ]
        mechanism = PyKCS11.CONCATENATE_DATA_AND_BASE_Mechanism(concData)
        derivedKey = self.session.deriveKey(self.baseKey, derivedKeyTemplate, mechanism)
        self.assertIsNotNone(derivedKey)

        # check derived key's value
        baseKeyValue = self.getCkaValue(self.baseKey)
        derivedKeyValue = self.getCkaValue(derivedKey)

        # match: check values
        self.assertSequenceEqual(concData + baseKeyValue, derivedKeyValue)

        # cleanup
        self.session.destroyObject(derivedKey)

    def test_deriveKey_CKM_XOR_BASE_AND_DATA(self):
        if self.manufacturer.startswith("SoftHSM"):
            self.skipTest("SoftHSM does not support CKM_XOR_BASE_AND_DATA")

        # generate data to xor with
        xorData = list(self.session.generateRandom(32))

        # xor key with data
        keyID = (0x22,)
        derivedKeyTemplate = self.genericKeyTemplate + [
            (PyKCS11.CKA_VALUE_LEN, 32),
            (PyKCS11.CKA_ID, keyID),
        ]
        mechanism = PyKCS11.XOR_BASE_AND_DATA_Mechanism(xorData)
        derivedKey = self.session.deriveKey(self.baseKey, derivedKeyTemplate, mechanism)
        self.assertIsNotNone(derivedKey)

        # check derived key's value
        baseKeyValue = self.getCkaValue(self.baseKey)
        derivedKeyValue = self.getCkaValue(derivedKey)
        expectedValue = map(lambda x, y: x ^ y, baseKeyValue, xorData)

        # match: check values
        self.assertSequenceEqual(list(expectedValue), derivedKeyValue)

        # cleanup
        self.session.destroyObject(derivedKey)

    def test_deriveKey_CKM_EXTRACT_KEY_FROM_KEY(self):
        if self.manufacturer.startswith("SoftHSM"):
            self.skipTest("SoftHSM does not support CKM_EXTRACT_KEY_FROM_KEY")

        # sample from the PKCS#11 specification 3.0, section 2.43.7
        # see https://docs.oasis-open.org/pkcs11/pkcs11-curr/v3.0/os/pkcs11-curr-v3.0-os.html#_Toc30061466
        #
        # We give an example of how this mechanism works.  Suppose a
        # token has a secret key with the 4-byte value 0x329F84A9.  We
        # will derive a 2-byte secret key from this key, starting at bit
        # position 21 (i.e., the value of the parameter to the
        # CKM_EXTRACT_KEY_FROM_KEY mechanism is 21).
        # 1.     We write the key’s value in binary: 0011 0010 1001 1111
        #    1000 0100 1010 1001.  We regard this binary string as
        #    holding the 32 bits of the key, labeled as b0, b1, …, b31.
        # 2.     We then extract 16 consecutive bits (i.e., 2 bytes)
        #    from this binary string, starting at bit b21.  We obtain
        #    the binary string 1001 0101 0010 0110.
        # 3.     The value of the new key is thus 0x9526.

        baseKeyTemplate = self.genericKeyTemplate + [
            (PyKCS11.CKA_DERIVE, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_VALUE, [0x32, 0x9F, 0x84, 0xA9]),
        ]

        # create a base key
        baseKey = self.session.createObject(baseKeyTemplate)

        expectedDerivedKeyValue = [0x95, 0x26]

        derivedKeyTemplate = self.genericKeyTemplate + [
            (PyKCS11.CKA_VALUE_LEN, len(expectedDerivedKeyValue))
        ]

        # extract bytes from the base key
        mechanism = PyKCS11.EXTRACT_KEY_FROM_KEY_Mechanism(21)
        derivedKey = self.session.deriveKey(baseKey, derivedKeyTemplate, mechanism)
        self.assertIsNotNone(derivedKey)

        derivedKeyValue = self.getCkaValue(derivedKey)

        # match: check derived key value
        self.assertSequenceEqual(expectedDerivedKeyValue, derivedKeyValue)

        self.session.destroyObject(baseKey)
        self.session.destroyObject(derivedKey)



# pylint: disable=missing-module-docstring
# pylint: disable=missing-class-docstring
# pylint: disable=missing-function-docstring

import unittest

from PyKCS11 import PyKCS11


class TestUtil(unittest.TestCase):
    def setUp(self):
        self.pkcs11 = PyKCS11.PyKCS11Lib()
        self.pkcs11.load()
        self.slot = self.pkcs11.getSlotList(tokenPresent=True)[0]
        self.manufacturerIDs = ("SoftHSM".ljust(32), "SoftHSM project".ljust(32))

    def test_getInfo(self):
        info = self.pkcs11.getInfo()

        # check the CK_UTF8CHAR to string convertion
        self.assertEqual(info.manufacturerID, "SoftHSM".ljust(32))
        self.assertEqual(info.libraryDescription, "Implementation of PKCS11".ljust(32))

        text = str(info)
        self.assertIsNotNone(text)
        d = info.to_dict()
        self.assertEqual(d["manufacturerID"], "SoftHSM".ljust(32))
        self.assertEqual(d["libraryDescription"], "Implementation of PKCS11".ljust(32))

    def test_getSlotInfo(self):
        info = self.pkcs11.getSlotInfo(self.slot)

        self.assertIn(info.manufacturerID, self.manufacturerIDs)

        text = str(info)
        self.assertIsNotNone(text)

    def test_getTokenInfo(self):
        info = self.pkcs11.getTokenInfo(self.slot)

        self.assertIn(info.manufacturerID, self.manufacturerIDs)

        text = str(info)
        self.assertIsNotNone(text)

    def test_getSessionInfo(self):
        self.session = self.pkcs11.openSession(self.slot, PyKCS11.CKF_SERIAL_SESSION)
        info = self.session.getSessionInfo()
        text = str(info)
        self.assertIsNotNone(text)

        d = info.to_dict()
        self.assertListEqual(d["flags"], ["CKF_SERIAL_SESSION"])
        self.assertEqual(d["ulDeviceError"], 0)

        self.session.closeSession()

    def test_getMechanismList(self):
        mechanisms = self.pkcs11.getMechanismList(self.slot)
        text = str(mechanisms)
        self.assertIsNotNone(text)

        # info for the first mechanism
        info = self.pkcs11.getMechanismInfo(self.slot, mechanisms[0])
        text = str(info)
        self.assertIsNotNone(text)



#!/usr/bin/env python3

"""
#   Copyright (C) 2015 Roman Pasechnik
#   Copyright (C) 2018 Ludovic Rousseau
#   Copyright (C) 2019 Atte Pellikka
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA.
"""


import binascii

import PyKCS11

# pylint: disable=duplicate-code

pkcs11 = PyKCS11.PyKCS11Lib()
pkcs11.load()  # define environment variable PYKCS11LIB=YourPKCS11Lib

# get 1st slot
slot = pkcs11.getSlotList(tokenPresent=True)[0]

session = pkcs11.openSession(slot, PyKCS11.CKF_SERIAL_SESSION | PyKCS11.CKF_RW_SESSION)
session.login("1234")

priv_search_tmpl = [
    (PyKCS11.CKA_CLASS, PyKCS11.CKO_PRIVATE_KEY),
    (PyKCS11.CKA_KEY_TYPE, PyKCS11.CKK_ECDSA),
]
pub_search_tmpl = [
    (PyKCS11.CKA_CLASS, PyKCS11.CKO_PUBLIC_KEY),
    (PyKCS11.CKA_KEY_TYPE, PyKCS11.CKK_ECDSA),
]

# "Hello world" in hex
toSign = "48656c6c6f20776f726c640d0a"
mechanism = PyKCS11.Mechanism(PyKCS11.CKM_ECDSA, None)

# find first private key and compute signature
privKey = session.findObjects(priv_search_tmpl)[0]
signature = session.sign(privKey, binascii.unhexlify(toSign), mechanism)
print(f"\nsignature: {binascii.hexlify(bytearray(signature))}")

# find first public key and verify signature
pubKey = session.findObjects(pub_search_tmpl)[0]
result = session.verify(pubKey, binascii.unhexlify(toSign), signature, mechanism)
print("\nVerified:", result)

# logout
session.logout()
session.closeSession()



#!/usr/bin/env python3

"""
#   Copyright (C) 2015 Roman Pasechnik
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA.
"""


import binascii

import PyKCS11

# pylint: disable=duplicate-code

pkcs11 = PyKCS11.PyKCS11Lib()
pkcs11.load()  # define environment variable PYKCS11LIB=YourPKCS11Lib

# get 1st slot
slot = pkcs11.getSlotList(tokenPresent=True)[0]

session = pkcs11.openSession(slot, PyKCS11.CKF_SERIAL_SESSION | PyKCS11.CKF_RW_SESSION)
session.login("1234")

# "Hello world" in hex
message = "48656c6c6f20776f726c640d0a"

# get first public and private keys
pubKey = session.findObjects([(PyKCS11.CKA_CLASS, PyKCS11.CKO_PUBLIC_KEY)])[0]
privKey = session.findObjects([(PyKCS11.CKA_CLASS, PyKCS11.CKO_PRIVATE_KEY)])[0]
enc = session.encrypt(pubKey, binascii.unhexlify(message))
dec = session.decrypt(privKey, enc)

print("\nmessage: " + message)
print(f"\nencrypted: {binascii.hexlify(bytearray(enc))}")
print(f"\ndecrypted: {bytearray(dec)}")

# logout
session.logout()
session.closeSession()



# pylint: disable=missing-module-docstring
# pylint: disable=missing-class-docstring
# pylint: disable=missing-function-docstring

import gc
import os
import platform
import shutil
import unittest
from pathlib import Path
from tempfile import TemporaryDirectory

from PyKCS11 import PyKCS11


class TestUtil(unittest.TestCase):
    def setUp(self):
        self.pkcs11 = PyKCS11.PyKCS11Lib()
        self.tmpdir = TemporaryDirectory()
        self.lib1_name = os.environ["PYKCS11LIB"]
        # create a tmp copy of the main lib
        # to use as a different library in tests
        self.lib2_name = str(Path(self.tmpdir.name) / Path(self.lib1_name).name)
        shutil.copy(self.lib1_name, self.tmpdir.name)

    def tearDown(self):
        del self.pkcs11
        if platform.system() != "Windows":
            self.tmpdir.cleanup()
        del self.tmpdir
        del self.lib1_name
        del self.lib2_name

    def openSession(self, lib):
        slot = lib.getSlotList(tokenPresent=True)[0]
        return lib.openSession(slot, PyKCS11.CKF_SERIAL_SESSION)

    def test_load(self):
        # create two instances with default library
        lib1 = PyKCS11.PyKCS11Lib().load()
        lib2 = PyKCS11.PyKCS11Lib().load()

        # expect two instances with the same library loaded
        self.assertTrue(hasattr(lib1, "pkcs11dll_filename"))
        self.assertTrue(hasattr(lib2, "pkcs11dll_filename"))
        self.assertEqual(len(lib1._loaded_libs), 1)
        self.assertEqual(len(lib2._loaded_libs), 1)

        self.openSession(lib1)

        # unload the first library
        del lib1
        gc.collect()

        # one instance remaining, the library is still in use
        self.openSession(lib2)
        self.assertEqual(len(lib2._loaded_libs), 1)

    def test_multiple_load(self):
        # load two different libraries
        lib1 = PyKCS11.PyKCS11Lib().load(self.lib1_name)
        lib2 = PyKCS11.PyKCS11Lib().load(self.lib2_name)

        # _loaded_libs is shared across all instances
        # check the value in self.pkcs11
        self.assertEqual(len(self.pkcs11._loaded_libs), 2)
        lib1 = PyKCS11.PyKCS11Lib()  # unload lib1
        self.assertEqual(len(self.pkcs11._loaded_libs), 1)
        lib2 = PyKCS11.PyKCS11Lib()  # unload lib2
        self.assertEqual(len(self.pkcs11._loaded_libs), 0)

    def test_invalid_load(self):
        # Library not found
        lib = "nolib"
        with self.assertRaises(PyKCS11.PyKCS11Error) as cm:
            self.pkcs11.load(lib)
        the_exception = cm.exception
        self.assertEqual(the_exception.value, -1)
        self.assertEqual(the_exception.text, lib)
        self.assertEqual(str(the_exception), f"Load ({lib})")
        self.assertEqual(len(self.pkcs11._loaded_libs), 0)

        # C_GetFunctionList() not found
        if platform.system() == "Linux":
            # GNU/Linux
            lib = "libc.so.6"
        elif platform.system() == "Darwin":
            # macOS
            lib = "/usr/lib/libSystem.B.dylib"
        else:
            # Windows
            lib = "WinSCard.dll"

        with self.assertRaises(PyKCS11.PyKCS11Error) as cm:
            self.pkcs11.load(lib)
        the_exception = cm.exception
        self.assertEqual(the_exception.value, -4)
        self.assertEqual(the_exception.text, lib)
        self.assertEqual(str(the_exception), f"C_GetFunctionList() not found ({lib})")
        self.assertEqual(len(self.pkcs11._loaded_libs), 0)

        # try to load the improper lib another time
        with self.assertRaises(PyKCS11.PyKCS11Error) as cm:
            self.pkcs11.load(lib)
        the_exception = cm.exception
        self.assertEqual(the_exception.value, -4)
        self.assertEqual(the_exception.text, lib)
        self.assertEqual(str(the_exception), f"C_GetFunctionList() not found ({lib})")
        self.assertEqual(len(self.pkcs11._loaded_libs), 0)

        # finally, load a valid library
        self.pkcs11.load()
        self.assertEqual(len(self.pkcs11._loaded_libs), 1)

    def test_specific_load(self):
        # load two different libraries sequentially
        self.pkcs11.load(self.lib1_name)
        self.pkcs11.load(self.lib2_name)

        # the second load should've unloaded the first library
        self.assertEqual(len(self.pkcs11._loaded_libs), 1)
        self.assertEqual(self.pkcs11.pkcs11dll_filename, self.lib2_name)

        # reload the first library
        self.pkcs11 = PyKCS11.PyKCS11Lib()
        self.pkcs11.load(self.lib1_name)

        # try to open a session
        self.assertIsNotNone(self.openSession(self.pkcs11))

    def test_unload(self):
        self.pkcs11.load().unload()
        # no pkcs11dll_filename should remain after unload
        self.assertIsNone(self.pkcs11.pkcs11dll_filename)

        self.pkcs11.load()
        self.openSession(self.pkcs11)
        # one library has been loaded
        self.assertEqual(len(self.pkcs11._loaded_libs), 1)
        self.assertIsNotNone(self.pkcs11.pkcs11dll_filename)

        self.pkcs11.unload()
        gc.collect()
        # manually unloaded the library using gc.collect()
        self.assertEqual(len(self.pkcs11._loaded_libs), 0)
        self.assertIsNone(self.pkcs11.pkcs11dll_filename)



# pylint: disable=missing-module-docstring
# pylint: disable=missing-class-docstring
# pylint: disable=missing-function-docstring

import unittest

from PyKCS11 import PyKCS11

# use OpenSC PKCS#11 Spy to check what PIN value is sent to the PKCS#11
# library


class TestUtil(unittest.TestCase):
    def setUp(self):
        self.pkcs11 = PyKCS11.PyKCS11Lib()
        self.pkcs11.load()
        self.slot = self.pkcs11.getSlotList(tokenPresent=True)[0]
        self.session = self.pkcs11.openSession(
            self.slot, PyKCS11.CKF_SERIAL_SESSION | PyKCS11.CKF_RW_SESSION
        )

    def tearDown(self):
        self.pkcs11.closeAllSessions(self.slot)
        del self.pkcs11

    def test_login(self):
        self.session.login("1234")
        self.session.logout()

    def test_wrong(self):
        with self.assertRaises(PyKCS11.PyKCS11Error) as cm:
            self.session.login("wrong PIN")

        the_exception = cm.exception
        self.assertEqual(the_exception.value, PyKCS11.CKR_PIN_INCORRECT)
        self.assertEqual(str(the_exception), "CKR_PIN_INCORRECT (0x000000A0)")

    def test_ckbytelist(self):
        pin = PyKCS11.ckbytelist("1234")
        self.session.login(pin)
        self.session.logout()

    def test_binary(self):
        with self.assertRaises(PyKCS11.PyKCS11Error) as cm:
            pin = PyKCS11.ckbytelist([1, 2, 3, 4])
            self.session.login(pin)

        the_exception = cm.exception
        self.assertEqual(the_exception.value, PyKCS11.CKR_PIN_INCORRECT)
        self.assertEqual(str(the_exception), "CKR_PIN_INCORRECT (0x000000A0)")

    def test_null(self):
        # SoftHSM2 does not support pinpad (pin = NULL)
        with self.assertRaises(PyKCS11.PyKCS11Error) as cm:
            self.session.login(None)

        the_exception = cm.exception
        self.assertEqual(the_exception.value, PyKCS11.CKR_ARGUMENTS_BAD)
        self.assertEqual(str(the_exception), "CKR_ARGUMENTS_BAD (0x00000007)")



#!/usr/bin/env python3

"""
#   Copyright (C) 2004 Midori (midori -- a-t -- paipai dot net)
#   Copyright (C) 2018 Ludovic Rousseau (ludovic.rousseau@free.fr)
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA.
"""

import os

import PyKCS11.LowLevel
from PyKCS11 import ckbytelist

# pylint: disable=duplicate-code

a = PyKCS11.LowLevel.CPKCS11Lib()
info = PyKCS11.LowLevel.CK_INFO()
slotInfo = PyKCS11.LowLevel.CK_SLOT_INFO()
lib = os.getenv("PYKCS11LIB")
if lib is None:
    raise ValueError("Define PYKCS11LIB")
session = PyKCS11.LowLevel.CK_SESSION_HANDLE()
sessionInfo = PyKCS11.LowLevel.CK_SESSION_INFO()
tokenInfo = PyKCS11.LowLevel.CK_TOKEN_INFO()
slotList = PyKCS11.LowLevel.ckulonglist()
pin = ckbytelist("1234")

print("Load of " + lib + ": " + str(a.Load(lib)))
print("C_GetInfo: " + hex(a.C_GetInfo(info)))
print("Library manufacturerID: " + info.GetManufacturerID())
del info

print("C_GetSlotList(NULL): " + hex(a.C_GetSlotList(0, slotList)))
print("\tAvailable Slots: " + str(len(slotList)))

for index, slot in enumerate(slotList):
    print("\tC_SlotInfo(): " + hex(a.C_GetSlotInfo(slot, slotInfo)))
    print(
        "\t\tSlot N."
        + str(index)
        + ": ID="
        + str(slot)
        + ", name='"
        + slotInfo.GetSlotDescription()
        + "'"
    )
    print(
        "\tC_OpenSession(): "
        + hex(
            a.C_OpenSession(
                slot,
                PyKCS11.LowLevel.CKF_SERIAL_SESSION | PyKCS11.LowLevel.CKF_RW_SESSION,
                session,
            )
        )
    )
    print("\t\tSession:" + str(session))
    print("\tC_GetSessionInfo(): " + hex(a.C_GetSessionInfo(session, sessionInfo)))
    print(
        "\t\tSessionInfo: state="
        + hex(sessionInfo.state)
        + ", flags="
        + hex(sessionInfo.flags)
    )

    print("\tC_GetTokenInfo(): " + hex(a.C_GetTokenInfo(slot, tokenInfo)))
    print(
        "\t\tTokenInfo: Label="
        + tokenInfo.GetLabel()
        + ", ManufacturerID="
        + tokenInfo.GetManufacturerID()
    )
    print(
        "\t\tTokenInfo: flags="
        + hex(tokenInfo.flags)
        + ", Model="
        + tokenInfo.GetModel()
    )

    print("\tC_Login(): " + hex(a.C_Login(session, PyKCS11.LowLevel.CKU_USER, pin)))
    print("\tC_Logout(): " + hex(a.C_Logout(session)))
    print("\tC_CloseSession(): " + hex(a.C_CloseSession(session)))

print(
    "C_OpenSession(): "
    + hex(
        a.C_OpenSession(
            slotList[0],
            PyKCS11.LowLevel.CKF_RW_SESSION | PyKCS11.LowLevel.CKF_SERIAL_SESSION,
            session,
        )
    )
)
print("C_Login(): " + hex(a.C_Login(session, PyKCS11.LowLevel.CKU_USER, pin)))

SearchResult = PyKCS11.LowLevel.ckulonglist(10)
SearchTemplate = PyKCS11.LowLevel.ckattrlist(2)
SearchTemplate[0].SetNum(PyKCS11.LowLevel.CKA_CLASS, PyKCS11.LowLevel.CKO_CERTIFICATE)
SearchTemplate[1].SetBool(PyKCS11.LowLevel.CKA_TOKEN, True)

print("C_FindObjectsInit: " + hex(a.C_FindObjectsInit(session, SearchTemplate)))
print("C_FindObjects: " + hex(a.C_FindObjects(session, SearchResult)))
print("C_FindObjectsFinal: " + hex(a.C_FindObjectsFinal(session)))

for x in SearchResult:
    print("object " + hex(x.value()))
    valTemplate = PyKCS11.LowLevel.ckattrlist(1)
    valTemplate[0].SetType(PyKCS11.LowLevel.CKA_ISSUER)
    # valTemplate[0].Reserve(128)
    print(
        "C_GetAttributeValue(): " + hex(a.C_GetAttributeValue(session, x, valTemplate))
    )
    print("CKA_ISSUER Len: ", valTemplate[0].GetLen())
    print(
        "C_GetAttributeValue(): " + hex(a.C_GetAttributeValue(session, x, valTemplate))
    )
    binval = list(valTemplate[0].GetBin())
    print("binval=", binval)
    binval[0] = 0
    valTemplate[0].SetBin(PyKCS11.LowLevel.CKA_ISSUER, binval)
    binval = valTemplate[0].GetBin()  # list(valTemplate[0].GetBin())
    print("binval[0]=", binval[0])
    binval[0] = 0

    print(
        "C_SetAttributeValue(): " + hex(a.C_SetAttributeValue(session, x, valTemplate))
    )

print("C_Logout(): " + hex(a.C_Logout(session)))
print("C_CloseSession(): " + hex(a.C_CloseSession(session)))
print("C_Finalize(): " + hex(a.C_Finalize()))
print(a.Unload())



# pylint: disable=missing-module-docstring
# pylint: disable=missing-class-docstring
# pylint: disable=missing-function-docstring

import gc
import unittest

from PyKCS11 import PyKCS11


class TestUtil(unittest.TestCase):
    def test_gc(self):
        res = []
        # we must use at least 2 sessions
        for _ in range(2):
            p11, session = self.createSession()
            res.append([p11, session])

        for p11, session in res:
            self.closeSession(session)
            del p11

            # force the call to __del__() on p11 now
            gc.collect()

    def createSession(self):
        pkcs11 = PyKCS11.PyKCS11Lib()
        pkcs11.load()

        slot = pkcs11.getSlotList(tokenPresent=True)[0]
        session = pkcs11.openSession(
            slot, PyKCS11.CKF_SERIAL_SESSION | PyKCS11.CKF_RW_SESSION
        )

        return (pkcs11, session)

    def closeSession(self, session):
        # this call generated a CKR_CRYPTOKI_NOT_INITIALIZED for the
        # second session because C_Finalize() was already called
        session.logout()



#!/usr/bin/env python3

"""
#   Copyright (C) 2006-2014 Ludovic Rousseau <ludovic.rousseau@free.fr>
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA.
"""


import PyKCS11

pin = "1234"
puk = "1234"

pkcs11 = PyKCS11.PyKCS11Lib()
pkcs11.load()
slot = pkcs11.getSlotList(tokenPresent=True)[0]
session = pkcs11.openSession(slot, PyKCS11.CKF_RW_SESSION)
session.login(puk, PyKCS11.CKU_SO)
session.initPin(pin)
session.logout()
session.closeSession()



# pylint: disable=missing-module-docstring
# pylint: disable=missing-class-docstring
# pylint: disable=missing-function-docstring

import os
import platform
import unittest

import PyKCS11.LowLevel
from PyKCS11 import ckbytelist


class TestUtil(unittest.TestCase):
    def test_LowLevel(self):
        a = PyKCS11.LowLevel.CPKCS11Lib()
        self.assertIsNotNone(a)

        # File not found
        self.assertEqual(a.Load("NoFile"), -1)

        # C_GetFunctionList() not found
        if platform.system() == "Linux":
            # GNU/Linux
            lib = "libc.so.6"
        elif platform.system() == "Darwin":
            # macOS
            lib = "/usr/lib/libSystem.B.dylib"
        else:
            # Windows
            lib = "WinSCard.dll"
        self.assertEqual(a.Load(lib), -4)

        info = PyKCS11.LowLevel.CK_INFO()
        self.assertIsNotNone(info)

        slotInfo = PyKCS11.LowLevel.CK_SLOT_INFO()
        self.assertIsNotNone(slotInfo)

        lib = os.getenv("PYKCS11LIB")
        if lib is None:
            raise Exception("Define PYKCS11LIB")

        session = PyKCS11.LowLevel.CK_SESSION_HANDLE()
        self.assertIsNotNone(session)

        sessionInfo = PyKCS11.LowLevel.CK_SESSION_INFO()
        self.assertIsNotNone(sessionInfo)

        tokenInfo = PyKCS11.LowLevel.CK_TOKEN_INFO()
        self.assertIsNotNone(tokenInfo)

        slotList = PyKCS11.LowLevel.ckulonglist()
        self.assertIsNotNone(slotList)

        longList = PyKCS11.LowLevel.ckulonglist(1)
        longList[0] = 42
        self.assertEqual(longList[0], 42)
        longList[0] = 0x80000000
        self.assertEqual(longList[0], 2147483648)
        longList[0] = 0xFFFFFFFF
        self.assertEqual(longList[0], 4294967295)
        # negative numbers are not allowed
        with self.assertRaises(TypeError):
            longList[0] = -1

        a.Load(lib)

        self.assertEqual(a.C_GetInfo(info), PyKCS11.LowLevel.CKR_OK)
        manufacturerID = info.GetManufacturerID()
        self.assertEqual(manufacturerID, "SoftHSM".ljust(32))
        del info

        a.C_GetSlotList(0, slotList)
        slot = slotList[0]

        self.assertEqual(a.C_GetSlotInfo(slot, slotInfo), PyKCS11.LowLevel.CKR_OK)

        self.assertEqual(
            a.C_OpenSession(
                slot,
                PyKCS11.LowLevel.CKF_SERIAL_SESSION | PyKCS11.LowLevel.CKF_RW_SESSION,
                session,
            ),
            PyKCS11.LowLevel.CKR_OK,
        )
        self.assertEqual(
            a.C_GetSessionInfo(session, sessionInfo), PyKCS11.LowLevel.CKR_OK
        )

        self.assertEqual(a.C_GetTokenInfo(slot, tokenInfo), PyKCS11.LowLevel.CKR_OK)
        label = tokenInfo.GetLabel()
        manufacturerID = tokenInfo.GetManufacturerID()
        flags = tokenInfo.flags
        model = tokenInfo.GetModel()

        pin = ckbytelist("1234")
        self.assertEqual(
            a.C_Login(session, PyKCS11.LowLevel.CKU_USER, pin), PyKCS11.LowLevel.CKR_OK
        )
        self.assertEqual(a.C_Logout(session), PyKCS11.LowLevel.CKR_OK)
        self.assertEqual(a.C_CloseSession(session), PyKCS11.LowLevel.CKR_OK)

        self.assertEqual(
            a.C_OpenSession(slotList[0], PyKCS11.LowLevel.CKF_SERIAL_SESSION, session),
            PyKCS11.LowLevel.CKR_OK,
        )
        self.assertEqual(
            a.C_Login(session, PyKCS11.LowLevel.CKU_USER, pin), PyKCS11.LowLevel.CKR_OK
        )

        SearchResult = PyKCS11.LowLevel.ckulonglist(10)
        SearchTemplate = PyKCS11.LowLevel.ckattrlist(2)
        SearchTemplate[0].SetNum(
            PyKCS11.LowLevel.CKA_CLASS, PyKCS11.LowLevel.CKO_CERTIFICATE
        )
        SearchTemplate[1].SetBool(PyKCS11.LowLevel.CKA_TOKEN, True)

        self.assertEqual(
            a.C_FindObjectsInit(session, SearchTemplate), PyKCS11.LowLevel.CKR_OK
        )
        self.assertEqual(
            a.C_FindObjects(session, SearchResult), PyKCS11.LowLevel.CKR_OK
        )
        self.assertEqual(a.C_FindObjectsFinal(session), PyKCS11.LowLevel.CKR_OK)

        for x in SearchResult:
            print("object: " + hex(x.value()))
            valTemplate = PyKCS11.LowLevel.ckattrlist(2)
            valTemplate[0].SetType(PyKCS11.LowLevel.CKA_LABEL)
            # valTemplate[0].Reserve(128)
            valTemplate[1].SetType(PyKCS11.LowLevel.CKA_CLASS)
            # valTemplate[1].Reserve(4)
            print(
                "C_GetAttributeValue(): "
                + hex(a.C_GetAttributeValue(session, x, valTemplate))
            )
            print(
                "CKA_LABEL Len: ",
                valTemplate[0].GetLen(),
                " CKA_CLASS Len: ",
                valTemplate[1].GetLen(),
            )
            print(
                "C_GetAttributeValue(): "
                + hex(a.C_GetAttributeValue(session, x, valTemplate))
            )
            print("\tCKO_CERTIFICATE: " + valTemplate[0].GetString())
            print("\tCKA_TOKEN: " + str(valTemplate[1].GetNum()))

        self.assertEqual(a.C_Logout(session), PyKCS11.LowLevel.CKR_OK)
        self.assertEqual(a.C_CloseSession(session), PyKCS11.LowLevel.CKR_OK)
        self.assertEqual(a.C_Finalize(), PyKCS11.LowLevel.CKR_OK)
        a.Unload()


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3

"""
#   Copyright (C) 2004 Midori (midori -- a-t -- paipai dot net)
#   Copyright (C) 2018 Ludovic Rousseau (ludovic.rousseau@free.fr)
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA.
"""

import os

import PyKCS11.LowLevel

a = PyKCS11.LowLevel.CPKCS11Lib()
info = PyKCS11.LowLevel.CK_INFO()
slotInfo = PyKCS11.LowLevel.CK_SLOT_INFO()
lib = os.getenv("PYKCS11LIB")
if lib is None:
    raise ValueError("Define PYKCS11LIB")
session = PyKCS11.LowLevel.CK_SESSION_HANDLE()
sessionInfo = PyKCS11.LowLevel.CK_SESSION_INFO()
tokenInfo = PyKCS11.LowLevel.CK_TOKEN_INFO()
slotList = PyKCS11.LowLevel.ckulonglist()
pin = PyKCS11.LowLevel.ckbytelist(b"123456")
puk = PyKCS11.LowLevel.ckbytelist(b"12345678")
Label = "PyKCS#11 Initialized Token      "

print("Load of " + lib + ": " + str(a.Load(lib)))
print("C_GetInfo: " + hex(a.C_GetInfo(info)))
print("Library manufacturerID: " + info.GetManufacturerID())
del info

print("C_GetSlotList(NULL): " + hex(a.C_GetSlotList(1, slotList)))
print("\tAvailable Slots: " + str(len(slotList)))

if len(slotList) != 0:
    print("\tC_SlotInfo(): " + hex(a.C_GetSlotInfo(slotList[0], slotInfo)))

    print("\tC_GetTokenInfo(): " + hex(a.C_GetTokenInfo(slotList[0], tokenInfo)))
    print(
        "\t\tTokenInfo: Label="
        + tokenInfo.GetLabel()
        + ", ManufacturerID="
        + tokenInfo.GetManufacturerID()
    )
    print(
        "\t\tTokenInfo: flags="
        + hex(tokenInfo.flags)
        + ", Model="
        + tokenInfo.GetModel()
    )
    print("\tC_InitToken(): " + hex(a.C_InitToken(slotList[0], puk, Label)))

    print("\tC_GetTokenInfo(): " + hex(a.C_GetTokenInfo(slotList[0], tokenInfo)))
    print(
        "\t\tTokenInfo: Label="
        + tokenInfo.GetLabel()
        + ", ManufacturerID="
        + tokenInfo.GetManufacturerID()
    )
    print(
        "\t\tTokenInfo: flags="
        + hex(tokenInfo.flags)
        + ", Model="
        + tokenInfo.GetModel()
    )

    print(
        "\tC_OpenSession(): "
        + hex(
            a.C_OpenSession(
                slotList[0],
                PyKCS11.LowLevel.CKF_SERIAL_SESSION | PyKCS11.LowLevel.CKF_RW_SESSION,
                session,
            )
        )
    )
    print("\t\tSession:" + str(session))
    print("\tC_GetSessionInfo(): " + hex(a.C_GetSessionInfo(session, sessionInfo)))
    print(
        "\t\tSessionInfo: state="
        + hex(sessionInfo.state)
        + ", flags="
        + hex(sessionInfo.flags)
    )

    print("\tC_Login(SO): " + hex(a.C_Login(session, PyKCS11.LowLevel.CKU_SO, puk)))
    print("\tC_InitPIN(): " + hex(a.C_InitPIN(session, pin)))
    print("\tC_Logout(SO): " + hex(a.C_Logout(session)))

    print("\tC_GetTokenInfo(): " + hex(a.C_GetTokenInfo(slotList[0], tokenInfo)))
    print(
        "\t\tTokenInfo: Label="
        + tokenInfo.GetLabel()
        + ", ManufacturerID="
        + tokenInfo.GetManufacturerID()
    )
    print(
        "\t\tTokenInfo: flags="
        + hex(tokenInfo.flags)
        + ", Model="
        + tokenInfo.GetModel()
    )

    print("C_Login(USER): " + hex(a.C_Login(session, PyKCS11.LowLevel.CKU_USER, pin)))
    print("C_Logout(USER): " + hex(a.C_Logout(session)))
    print("C_CloseSession(): " + hex(a.C_CloseSession(session)))

print("C_Finalize(): " + hex(a.C_Finalize()))
print(a.Unload())



# pylint: disable=missing-module-docstring
# pylint: disable=missing-class-docstring
# pylint: disable=missing-function-docstring

import binascii
import unittest

from asn1crypto.core import OctetString
from asn1crypto.keys import ECDomainParameters, NamedCurve, PrivateKeyAlgorithmId

from PyKCS11 import PyKCS11


class TestUtil(unittest.TestCase):
    def setUp(self):
        self.pkcs11 = PyKCS11.PyKCS11Lib()
        self.pkcs11.load()

        # get SoftHSM major version
        self.SoftHSMversion = self.pkcs11.getInfo().libraryVersion[0]

        self.slot = self.pkcs11.getSlotList(tokenPresent=True)[0]
        self.session = self.pkcs11.openSession(
            self.slot, PyKCS11.CKF_SERIAL_SESSION | PyKCS11.CKF_RW_SESSION
        )
        self.session.login("1234")

        # Select the curve to be used for the keys
        curve = "secp256r1"

        # Setup the domain parameters, unicode conversion needed
        # for the curve string
        domain_params = ECDomainParameters(name="named", value=NamedCurve(curve))
        ec_params = domain_params.dump()

        keyID = (0x22,)
        label = "test"

        ec_public_tmpl = [
            (PyKCS11.CKA_CLASS, PyKCS11.CKO_PUBLIC_KEY),
            (PyKCS11.CKA_PRIVATE, PyKCS11.CK_FALSE),
            (PyKCS11.CKA_TOKEN, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_ENCRYPT, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_VERIFY, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_WRAP, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_KEY_TYPE, PyKCS11.CKK_ECDSA),
            (PyKCS11.CKA_EC_PARAMS, ec_params),
            (PyKCS11.CKA_LABEL, label),
            (PyKCS11.CKA_ID, keyID),
        ]

        ec_priv_tmpl = [
            (PyKCS11.CKA_CLASS, PyKCS11.CKO_PRIVATE_KEY),
            (PyKCS11.CKA_KEY_TYPE, PyKCS11.CKK_ECDSA),
            (PyKCS11.CKA_TOKEN, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_SENSITIVE, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_DECRYPT, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_SIGN, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_UNWRAP, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_LABEL, label),
            (PyKCS11.CKA_ID, keyID),
        ]

        if self.SoftHSMversion < 2:
            self.skipTest("ECDSA only supported by SoftHSM >= 2")

        (self.pubKey, self.privKey) = self.session.generateKeyPair(
            ec_public_tmpl, ec_priv_tmpl, mecha=PyKCS11.MechanismECGENERATEKEYPAIR
        )
        self.assertIsNotNone(self.pubKey)
        self.assertIsNotNone(self.privKey)

        # test CK_OBJECT_HANDLE.__repr__()
        text = str(self.pubKey)
        self.assertIsNotNone(text)

    def tearDown(self):
        self.session.destroyObject(self.pubKey)
        self.session.destroyObject(self.privKey)

        self.session.logout()
        self.pkcs11.closeAllSessions(self.slot)
        del self.pkcs11

    def test_sign_integer(self):
        toSign = 1234567890
        mecha = PyKCS11.Mechanism(PyKCS11.CKM_ECDSA, None)

        # sign/verify
        with self.assertRaises(PyKCS11.PyKCS11Error) as cm:
            self.session.sign(self.privKey, toSign, mecha)
        # should return PyKCS11.PyKCS11Error: Unknown format (<class 'int'>)
        self.assertEqual(cm.exception.value, -3)

    def test_sign_text(self):
        toSign = "Hello World!"
        mecha = PyKCS11.Mechanism(PyKCS11.CKM_ECDSA, None)

        # sign/verify
        signature = self.session.sign(self.privKey, toSign, mecha)

        result = self.session.verify(self.pubKey, toSign, signature, mecha)

        self.assertTrue(result)


class TestEDDSA(unittest.TestCase):
    def setUp(self):
        self.pkcs11 = PyKCS11.PyKCS11Lib()
        self.pkcs11.load()

        self.manufacturer = self.pkcs11.getInfo().manufacturerID
        self.slot = self.pkcs11.getSlotList(tokenPresent=True)[0]

        for m in ["CKM_EC_EDWARDS_KEY_PAIR_GEN", "CKM_EDDSA"]:
            if not m in self.pkcs11.getMechanismList(self.slot):
                self.pkcs11.unload()
                self.skipTest(f"{m} is not supported by the token")

        self.session = self.pkcs11.openSession(
            self.slot, PyKCS11.CKF_SERIAL_SESSION | PyKCS11.CKF_RW_SESSION
        )
        self.session.login("1234")

        self.ec_base_tmpl = [
            (PyKCS11.CKA_TOKEN, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_KEY_TYPE, PyKCS11.CKK_EC_EDWARDS),
            (PyKCS11.CKA_LABEL, "test"),
            (PyKCS11.CKA_ID, (0x33,)),
        ]

        self.ec_public_tmpl = self.ec_base_tmpl + [
            (PyKCS11.CKA_CLASS, PyKCS11.CKO_PUBLIC_KEY),
            (PyKCS11.CKA_PRIVATE, PyKCS11.CK_FALSE),
            (PyKCS11.CKA_ENCRYPT, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_VERIFY, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_WRAP, PyKCS11.CK_TRUE),
        ]

        self.ec_priv_tmpl = self.ec_base_tmpl + [
            (PyKCS11.CKA_CLASS, PyKCS11.CKO_PRIVATE_KEY),
            (PyKCS11.CKA_SENSITIVE, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_DECRYPT, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_SIGN, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_UNWRAP, PyKCS11.CK_TRUE),
        ]

        # see https://docs.oasis-open.org/pkcs11/pkcs11-curr/v3.0/os/pkcs11-curr-v3.0-os.html#_Toc30061191
        self.schemes = {
            # scheme     : ("curve", phFlag)
            "Ed25519": ("ed25519", None),
            "Ed25519ctx": ("ed25519", False),
            "Ed25519ph": ("ed25519", True),
            "Ed448": ("ed448", False),
            "Ed448ph": ("ed448", True),
        }

    def tearDown(self):
        self.session.logout()
        self.pkcs11.closeAllSessions(self.slot)
        self.pkcs11.unload()
        del self.pkcs11

    def _get_eddsa_mechanism(self, phf=None, ctx=None):
        if phf is None:
            return PyKCS11.EDDSA_Mechanism()
        elif ctx is None:
            return PyKCS11.EDDSA_Mechanism(phFlag=phf)
        else:
            return PyKCS11.EDDSA_Mechanism(phFlag=phf, contextData=ctx)

    def _generate_edwards_key_pair(self, curve):
        # Setup the domain parameters, unicode conversion needed
        # for the curve string
        ec_params = PrivateKeyAlgorithmId(curve).dump()
        ec_public_tmpl = self.ec_public_tmpl + [(PyKCS11.CKA_EC_PARAMS, ec_params)]

        (pubKey, privKey) = self.session.generateKeyPair(
            ec_public_tmpl,
            self.ec_priv_tmpl,
            mecha=PyKCS11.Mechanism(PyKCS11.CKM_EC_EDWARDS_KEY_PAIR_GEN, None),
        )
        self.assertIsNotNone(pubKey)
        self.assertIsNotNone(privKey)

        # test CK_OBJECT_HANDLE.__repr__()
        text = str(pubKey)
        self.assertIsNotNone(text)
        return (pubKey, privKey)

    def test_sign_integer(self):
        toSign = 1234567890
        mecha = PyKCS11.EDDSA_Mechanism()
        (pubKey, privKey) = self._generate_edwards_key_pair("ed25519")

        # sign/verify
        with self.assertRaises(PyKCS11.PyKCS11Error) as cm:
            self.session.sign(privKey, toSign, mecha)
        # should return PyKCS11.PyKCS11Error: Unknown format (<class 'int'>)
        self.assertEqual(cm.exception.value, -3)

        self.session.destroyObject(pubKey)
        self.session.destroyObject(privKey)

    def test_sign_text(self):
        toSign = ctxData = "Hello World!"
        schemes = [
            # Ed25519
            ("ed25519", None, None),
            # Ed25519ctx
            ("ed25519", False, None),
            ("ed25519", False, ctxData),
            # Ed25519ph
            ("ed25519", True, None),
            ("ed25519", True, ctxData),
            # Ed448
            ("ed448", False, None),
            ("ed448", False, ctxData),
            # Ed448ph
            ("ed448", True, None),
            ("ed448", True, ctxData),
        ]

        for curve, phflag, context in schemes:
            with self.subTest(curve=curve, phflag=phflag, context=context):
                mech = self._get_eddsa_mechanism(phflag, context)
                (pubKey, privKey) = self._generate_edwards_key_pair(curve)
                # sign/verify
                signature = self.session.sign(privKey, toSign, mech)

                self.assertTrue(self.session.verify(pubKey, toSign, signature, mech))

                self.session.destroyObject(pubKey)
                self.session.destroyObject(privKey)

    def test_sign_test_vectors(self):
        # test vectors from RFC 8032, Sec. 7
        test_vectors = [
            # Ed25519
            {
                "scheme": "Ed25519",
                "secret": "c5aa8df43f9f837bedb7442f31dcb7b1"
                "66d38535076f094b85ce3a2e0b4458f7",
                "public": "fc51cd8e6218a1a38da47ed00230f058"
                "0816ed13ba3303ac5deb911548908025",
                "message": "af82",
                "signature": "6291d657deec24024827e69c3abe01a3"
                "0ce548a284743a445e3680d7db5ac3ac"
                "18ff9b538d16f290ae67f760984dc659"
                "4a7c15e9716ed28dc027beceea1ec40a",
            },
            # Ed25519ctx
            {
                "scheme": "Ed25519ctx",
                "secret": "0305334e381af78f141cb666f6199f57"
                "bc3495335a256a95bd2a55bf546663f6",
                "public": "dfc9425e4f968f7f0c29f0259cf5f9ae"
                "d6851c2bb4ad8bfb860cfee0ab248292",
                "message": "f726936d19c800494e3fdaff20b276a8",
                "context": "666f6f",
                "signature": "55a4cc2f70a54e04288c5f4cd1e45a7b"
                "b520b36292911876cada7323198dd87a"
                "8b36950b95130022907a7fb7c4e9b2d5"
                "f6cca685a587b4b21f4b888e4e7edb0d",
            },
            # Ed25519ph
            {
                "scheme": "Ed25519ph",
                "secret": "833fe62409237b9d62ec77587520911e"
                "9a759cec1d19755b7da901b96dca3d42",
                "public": "ec172b93ad5e563bf4932c70e1245034"
                "c35467ef2efd4d64ebf819683467e2bf",
                "message": "616263",
                "signature": "98a70222f0b8121aa9d30f813d683f80"
                "9e462b469c7ff87639499bb94e6dae41"
                "31f85042463c2a355a2003d062adf5aa"
                "a10b8c61e636062aaad11c2a26083406",
            },
            # Ed448
            {
                "scheme": "Ed448",
                "secret": "c4eab05d357007c632f3dbb48489924d"
                "552b08fe0c353a0d4a1f00acda2c463a"
                "fbea67c5e8d2877c5e3bc397a659949e"
                "f8021e954e0a12274e",
                "public": "43ba28f430cdff456ae531545f7ecd0a"
                "c834a55d9358c0372bfa0c6c6798c086"
                "6aea01eb00742802b8438ea4cb82169c"
                "235160627b4c3a9480",
                "message": "03",
                "context": "666f6f",
                "signature": "d4f8f6131770dd46f40867d6fd5d5055"
                "de43541f8c5e35abbcd001b32a89f7d2"
                "151f7647f11d8ca2ae279fb842d60721"
                "7fce6e042f6815ea000c85741de5c8da"
                "1144a6a1aba7f96de42505d7a7298524"
                "fda538fccbbb754f578c1cad10d54d0d"
                "5428407e85dcbc98a49155c13764e66c"
                "3c00",
            },
            # Ed448ph
            {
                "scheme": "Ed448ph",
                "secret": "833fe62409237b9d62ec77587520911e"
                "9a759cec1d19755b7da901b96dca3d42"
                "ef7822e0d5104127dc05d6dbefde69e3"
                "ab2cec7c867c6e2c49",
                "public": "259b71c19f83ef77a7abd26524cbdb31"
                "61b590a48f7d17de3ee0ba9c52beb743"
                "c09428a131d6b1b57303d90d8132c276"
                "d5ed3d5d01c0f53880",
                "message": "616263",
                "context": "666f6f",
                "signature": "c32299d46ec8ff02b54540982814dce9"
                "a05812f81962b649d528095916a2aa48"
                "1065b1580423ef927ecf0af5888f90da"
                "0f6a9a85ad5dc3f280d91224ba9911a3"
                "653d00e484e2ce232521481c8658df30"
                "4bb7745a73514cdb9bf3e15784ab7128"
                "4f8d0704a608c54a6b62d97beb511d13"
                "2100",
            },
        ]

        def verify_test_vector(t):
            curve = self.schemes[t["scheme"]][0]
            phflag = self.schemes[t["scheme"]][1]
            ec_params = PrivateKeyAlgorithmId(curve).dump()
            ec_point = OctetString(binascii.unhexlify(t["public"])).dump()

            ec_public_tmpl = self.ec_public_tmpl + [
                (PyKCS11.CKA_EC_PARAMS, ec_params),
                (PyKCS11.CKA_EC_POINT, ec_point),
            ]
            ec_priv_tmpl = self.ec_priv_tmpl + [
                (PyKCS11.CKA_EC_PARAMS, ec_params),
                (PyKCS11.CKA_VALUE, binascii.unhexlify(t["secret"])),
            ]

            prk = self.session.createObject(ec_priv_tmpl)
            pbk = self.session.createObject(ec_public_tmpl)

            if "context" in t:
                context = binascii.unhexlify(t["context"])
                mech = self._get_eddsa_mechanism(phflag, context)
            else:
                mech = self._get_eddsa_mechanism(phflag)

            message = binascii.unhexlify(t["message"])
            signature = self.session.sign(prk, message, mech)
            signatureStr = binascii.hexlify(bytes(signature)).decode("ascii")
            self.assertSequenceEqual(signatureStr, t["signature"])
            self.assertTrue(self.session.verify(pbk, message, signature, mech))

            self.session.destroyObject(prk)
            self.session.destroyObject(pbk)

        for t in test_vectors:
            with self.subTest(t=t):
                if (
                    self.manufacturer.startswith("SoftHSM")
                    and not "Ed25519" == t["scheme"]
                ):
                    # SoftHSM just silently ignores CK_EDDSA_PARAMS,
                    # which means that the only supported scheme is Ed25519
                    self.skipTest(
                        f"SoftHSM does not support {t['scheme']} signature scheme"
                    )
                verify_test_vector(t)



# pylint: disable=missing-module-docstring
# pylint: disable=missing-class-docstring
# pylint: disable=missing-function-docstring

import unittest

from PyKCS11 import PyKCS11

# those shortcuts make the testing code more readable
CK_FALSE = PyKCS11.CK_FALSE
CK_TRUE = PyKCS11.CK_TRUE


class TestUtil(unittest.TestCase):
    def setUp(self):
        self.pkcs11 = PyKCS11.PyKCS11Lib()
        self.pkcs11.load()

        # get SoftHSM major version
        self.SoftHSMversion = self.pkcs11.getInfo().libraryVersion[0]

        self.slot = self.pkcs11.getSlotList(tokenPresent=True)[0]

        self.session = self.pkcs11.openSession(
            self.slot, PyKCS11.CKF_SERIAL_SESSION | PyKCS11.CKF_RW_SESSION
        )
        self.session.login("1234")

    def tearDown(self):
        self.session.logout()
        self.pkcs11.closeAllSessions(self.slot)
        del self.pkcs11

    def test_objects(self):
        if self.SoftHSMversion < 2:
            self.skipTest("generateKey() only supported by SoftHSM >= 2")

        AESKeyTemplate = [
            (PyKCS11.CKA_CLASS, PyKCS11.CKO_SECRET_KEY),
            (PyKCS11.CKA_KEY_TYPE, PyKCS11.CKK_AES),
            (PyKCS11.CKA_TOKEN, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_PRIVATE, PyKCS11.CK_FALSE),
            (PyKCS11.CKA_ENCRYPT, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_DECRYPT, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_SIGN, PyKCS11.CK_FALSE),
            (PyKCS11.CKA_VERIFY, PyKCS11.CK_FALSE),
            (PyKCS11.CKA_VALUE_LEN, 32),
            (PyKCS11.CKA_LABEL, "TestAESKey"),
            (PyKCS11.CKA_ID, (0x01,)),
        ]

        # generate AES key
        AESKey = self.session.generateKey(AESKeyTemplate)
        self.assertIsNotNone(AESKey)

        # find the first secret key
        symKey = self.session.findObjects(
            [(PyKCS11.CKA_CLASS, PyKCS11.CKO_SECRET_KEY)]
        )[0]

        # test object handle
        text = str(symKey)
        self.assertIsNotNone(text)

        # test createObject()
        template = [(PyKCS11.CKA_CLASS, PyKCS11.CKO_DATA), (PyKCS11.CKA_LABEL, "data")]
        handle = self.session.createObject(template)
        self.assertIsNotNone(handle)

        self.session.destroyObject(handle)

        # test getAttributeValue

        # attributes as define by AESKeyTemplate
        all_attributes = [
            PyKCS11.CKA_CLASS,
            PyKCS11.CKA_KEY_TYPE,
            PyKCS11.CKA_TOKEN,
            PyKCS11.CKA_LABEL,
            PyKCS11.CKA_ID,
        ]

        values = self.session.getAttributeValue(AESKey, all_attributes)
        self.assertEqual(
            values,
            [
                PyKCS11.CKO_SECRET_KEY,
                PyKCS11.CKK_AES,
                PyKCS11.CK_TRUE,
                "TestAESKey",
                (0x01,),
            ],
        )

        # clean up
        self.session.destroyObject(AESKey)

        template = [(PyKCS11.CKA_HW_FEATURE_TYPE, PyKCS11.CKH_USER_INTERFACE)]
        o = self.session.findObjects(template)
        self.assertListEqual(o, [])

    def test_BoolAttributes(self):
        # dictionary of attributes expected to be bool and their expected values
        boolAttributes = {
            PyKCS11.CKA_TOKEN: PyKCS11.CK_FALSE,
            PyKCS11.CKA_PRIVATE: PyKCS11.CK_FALSE,
            # The attributes below are defaulted to CK_TRUE
            # ( according to the PKCS#11 standard )
            PyKCS11.CKA_MODIFIABLE: PyKCS11.CK_TRUE,
            PyKCS11.CKA_COPYABLE: PyKCS11.CK_TRUE,
            PyKCS11.CKA_DESTROYABLE: PyKCS11.CK_TRUE,
        }

        CkoDataTemplate = [
            (PyKCS11.CKA_CLASS, PyKCS11.CKO_DATA),
            (PyKCS11.CKA_TOKEN, PyKCS11.CK_FALSE),
            (PyKCS11.CKA_PRIVATE, PyKCS11.CK_FALSE),
            (PyKCS11.CKA_LABEL, "TestData"),
        ]

        # create a CKO_DATA object
        ckoData = self.session.createObject(CkoDataTemplate)
        self.assertIsNotNone(ckoData)

        attrValues = self.session.getAttributeValue(
            ckoData, list(boolAttributes.keys())
        )

        # check that attributes are of bool type
        # and have expected values
        for i, attr in enumerate(boolAttributes):
            self.assertIsInstance(attrValues[i], bool)
            self.assertEqual(attrValues[i], boolAttributes[attr])

        # clean up
        self.session.destroyObject(ckoData)


class TestGetSetAttributeValues(unittest.TestCase):

    def setUp(self) -> None:

        self.pkcs11 = PyKCS11.PyKCS11Lib()
        self.pkcs11.load()

        # get SoftHSM major version
        self.SoftHSMversion = self.pkcs11.getInfo().libraryVersion[0]
        if self.SoftHSMversion < 2:
            self.skipTest("generateKey() only supported by SoftHSM >= 2")

        self.slot = self.pkcs11.getSlotList(tokenPresent=True)[0]

        self.session = self.pkcs11.openSession(
            self.slot, PyKCS11.CKF_SERIAL_SESSION | PyKCS11.CKF_RW_SESSION
        )
        self.session.login("1234")

        AESKeyTemplate = [
            (PyKCS11.CKA_CLASS, PyKCS11.CKO_SECRET_KEY),
            (PyKCS11.CKA_KEY_TYPE, PyKCS11.CKK_AES),
            (PyKCS11.CKA_TOKEN, CK_TRUE),
            (PyKCS11.CKA_PRIVATE, CK_FALSE),
            (PyKCS11.CKA_ENCRYPT, CK_TRUE),
            (PyKCS11.CKA_DECRYPT, CK_TRUE),
            (PyKCS11.CKA_SIGN, CK_FALSE),
            (PyKCS11.CKA_VERIFY, CK_FALSE),
            (PyKCS11.CKA_VALUE_LEN, 32),
            (PyKCS11.CKA_LABEL, "TestAESKey"),
            (PyKCS11.CKA_ID, (0x01,)),
        ]

        # generate AES key
        self.AESKey = self.session.generateKey(AESKeyTemplate)
        self.assertIsNotNone(self.AESKey)

    def tearDown(self):
        self.session.destroyObject(self.AESKey)
        self.session.logout()
        self.pkcs11.closeAllSessions(self.slot)
        del self.pkcs11

    def test_getAttributeValue(self):

        # attributes as defined by AESKeyTemplate in setUp
        all_attributes = [
            PyKCS11.CKA_CLASS,
            PyKCS11.CKA_KEY_TYPE,
            PyKCS11.CKA_TOKEN,
            PyKCS11.CKA_LABEL,
            PyKCS11.CKA_ID,
        ]

        values = self.session.getAttributeValue(self.AESKey, all_attributes)
        self.assertEqual(
            values,
            [
                PyKCS11.CKO_SECRET_KEY,
                PyKCS11.CKK_AES,
                CK_TRUE,
                "TestAESKey",
                (0x01,),
            ],
        )

    def test_setAttributeValue_with_single_binary_attribute(self):
        # test setAttributeValue with a binary attribute
        _ATTR = PyKCS11.CKA_SIGN  # which attribute to test with. use a binary attribute

        old_state = self.session.getAttributeValue(self.AESKey, [_ATTR])[0]
        new_state = CK_TRUE if old_state == CK_FALSE else CK_FALSE  # switch the state

        # pylint: disable=assignment-from-no-return
        rv = self.session.setAttributeValue(self.AESKey, [(_ATTR, new_state)])
        assert rv is None

        # test to see if object is really modified
        test_state = self.session.getAttributeValue(self.AESKey, [_ATTR])[0]
        assert test_state == new_state
        assert test_state != old_state

    def test_setAttributeValue_with_a_list_of_attributes(self):

        # which binary attributes to flip?
        attributes_to_switch = [
            PyKCS11.CKA_SIGN,
            PyKCS11.CKA_ENCRYPT,
            PyKCS11.CKA_DECRYPT,
            PyKCS11.CKA_VERIFY,
            PyKCS11.CKA_WRAP,
            PyKCS11.CKA_UNWRAP,
        ]

        old_attributes = self.session.getAttributeValue(
            self.AESKey, attributes_to_switch
        )

        flipped_attributes = []
        for i, attr in enumerate(attributes_to_switch):
            new_value = CK_TRUE if old_attributes[i] == CK_FALSE else CK_FALSE
            flipped_attributes.append((attr, new_value))

        # pylint: disable=assignment-from-no-return
        rv = self.session.setAttributeValue(self.AESKey, flipped_attributes)
        assert rv is None

        new_attributes = self.session.getAttributeValue(
            self.AESKey, attributes_to_switch
        )
        for new, old in zip(new_attributes, old_attributes):
            assert new != old
            assert (new == CK_TRUE and old == CK_FALSE) or (
                new == CK_FALSE and old == CK_TRUE
            )

    def test_setAttributeValue_with_label_attribute(self):
        # test setAttributeValue with the text field `CKA_Label` by appending some text

        old_label = self.session.getAttributeValue(self.AESKey, [PyKCS11.CKA_LABEL])[0]
        new_label = old_label + "-mod"
        self.session.setAttributeValue(self.AESKey, [(PyKCS11.CKA_LABEL, new_label)])
        test_label = self.session.getAttributeValue(self.AESKey, [PyKCS11.CKA_LABEL])[0]

        assert new_label != old_label
        assert test_label == new_label
        assert test_label != old_label



#!/usr/bin/env python3


import os
import sys


def get_PYKCS11LIB():
    # already configured
    if "PYKCS11LIB" in os.environ:
        return os.environ["PYKCS11LIB"]

    if "PKCS11SPY" in os.environ:
        # use OpenSC PKCS#11 spy if PKCS11SPY is defined as the PKCS#11 lib
        # to use
        LIBS = [
            "/usr/local/lib/pkcs11-spy.so",  # macOS or local build
            "/usr/lib/x86_64-linux-gnu/pkcs11-spy.so",  # Debian amd64
            "/usr/lib64/pkcs11/pkcs11-spy.so",  # Fedora Linux
        ]
    else:
        if sys.maxsize > 2**32:
            # 64-bits
            WINDOWS_SOFTHSM = "c:/SoftHSM2/lib/softhsm2-x64.dll"
        else:
            # 32-bits
            WINDOWS_SOFTHSM = "c:/SoftHSM2/lib/softhsm2.dll"
        # use SoftHSM2 or SoftHSM1
        LIBS = [
            "/usr/local/lib/softhsm/libsofthsm2.so",  # macOS or local build
            "/opt/homebrew/lib/softhsm/libsofthsm2.so",  # macOS arm64
            "/usr/lib/softhsm/libsofthsm2.so",  # Debian libsofthsm2
            "/usr/lib/softhsm/libsofthsm.so",  # Debian libsofthsm
            "/usr/lib/libsofthsm.so",  # Ubuntu 12.04 libsofthsm
            "/usr/lib/x86_64-linux-gnu/softhsm/libsofthsm2.so",  # Ubuntu 16.04 libsofthsm2
            "/usr/lib64/pkcs11/libsofthsm2.so",  # Fedora Linux
            WINDOWS_SOFTHSM,  # Windows
        ]

    for lib in LIBS:
        if os.path.isfile(lib):
            return lib

    raise Exception("PYKCS11LIB is not defined. No SoftHSM library found?")


if __name__ == "__main__":
    lib = get_PYKCS11LIB()
    print(f"PYKCS11LIB={lib}")



# pylint: disable=missing-module-docstring
# pylint: disable=missing-class-docstring
# pylint: disable=missing-function-docstring

import unittest

from PyKCS11 import PyKCS11


class TestUtil(unittest.TestCase):
    def setUp(self):
        self.pkcs11 = PyKCS11.PyKCS11Lib()
        self.pkcs11.load()

        # get SoftHSM major version
        self.SoftHSMversion = self.pkcs11.getInfo().libraryVersion[0]

        self.slot = self.pkcs11.getSlotList(tokenPresent=True)[0]
        self.session = self.pkcs11.openSession(
            self.slot, PyKCS11.CKF_SERIAL_SESSION | PyKCS11.CKF_RW_SESSION
        )
        self.session.login("1234")

    def tearDown(self):
        self.session.logout()
        self.pkcs11.closeAllSessions(self.slot)
        del self.pkcs11

    def test_wrapKey(self):
        keyID = (0x01,)
        AESKeyTemplate = [
            (PyKCS11.CKA_CLASS, PyKCS11.CKO_SECRET_KEY),
            (PyKCS11.CKA_KEY_TYPE, PyKCS11.CKK_AES),
            (PyKCS11.CKA_TOKEN, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_PRIVATE, PyKCS11.CK_FALSE),
            (PyKCS11.CKA_ENCRYPT, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_DECRYPT, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_SIGN, PyKCS11.CK_FALSE),
            (PyKCS11.CKA_EXTRACTABLE, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_VERIFY, PyKCS11.CK_FALSE),
            (PyKCS11.CKA_VALUE_LEN, 32),
            (PyKCS11.CKA_LABEL, "TestAESKey"),
            (PyKCS11.CKA_ID, keyID),
        ]

        if self.SoftHSMversion < 2:
            self.skipTest("generateKey() only supported by SoftHSM >= 2")

        self.wrapKey = self.session.generateKey(AESKeyTemplate)
        self.assertIsNotNone(self.wrapKey)

        keyID = (0x02,)
        # make the key extractable
        AESKeyTemplate.append((PyKCS11.CKA_EXTRACTABLE, PyKCS11.CK_TRUE))

        self.AESKey = self.session.generateKey(AESKeyTemplate)
        self.assertIsNotNone(self.AESKey)

        # buffer of 32 bytes 0x42
        DataIn = [42] * 32

        mechanism = PyKCS11.Mechanism(PyKCS11.CKM_AES_ECB)
        DataOut = self.session.encrypt(self.AESKey, DataIn, mechanism)
        # print("DataOut", DataOut)

        DataCheck = self.session.decrypt(self.AESKey, DataOut, mechanism)
        # print("DataCheck:", DataCheck)

        # check we can encrypt/decrypt with the AES key
        self.assertSequenceEqual(DataIn, DataCheck)

        # wrap using CKM_AES_KEY_WRAP
        mechanismWrap = PyKCS11.Mechanism(PyKCS11.CKM_AES_KEY_WRAP)
        wrapped = self.session.wrapKey(self.wrapKey, self.AESKey, mechanismWrap)
        self.assertIsNotNone(wrapped)

        # destroy the original key
        self.session.destroyObject(self.AESKey)

        # unwrap
        template = [
            (PyKCS11.CKA_CLASS, PyKCS11.CKO_SECRET_KEY),
            (PyKCS11.CKA_KEY_TYPE, PyKCS11.CKK_AES),
            (PyKCS11.CKA_TOKEN, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_PRIVATE, PyKCS11.CK_FALSE),
            (PyKCS11.CKA_ENCRYPT, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_DECRYPT, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_SIGN, PyKCS11.CK_FALSE),
            (PyKCS11.CKA_VERIFY, PyKCS11.CK_FALSE),
        ]
        unwrapped = self.session.unwrapKey(
            self.wrapKey, wrapped, template, mechanismWrap
        )
        self.assertIsNotNone(unwrapped)

        DataCheck = self.session.decrypt(unwrapped, DataOut, mechanism)
        # print("DataCheck:", DataCheck)

        # check we can decrypt with the unwrapped AES key
        self.assertSequenceEqual(DataIn, DataCheck)

        # cleanup
        self.session.destroyObject(unwrapped)

        self.session.destroyObject(self.wrapKey)

    def test_wrapKey_OAEP(self):
        if self.SoftHSMversion < 2:
            self.skipTest("generateKey() only supported by SoftHSM >= 2")

        keyID = (0x22,)
        pubTemplate = [
            (PyKCS11.CKA_CLASS, PyKCS11.CKO_PUBLIC_KEY),
            (PyKCS11.CKA_TOKEN, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_PRIVATE, PyKCS11.CK_FALSE),
            (PyKCS11.CKA_MODULUS_BITS, 0x0400),
            (PyKCS11.CKA_PUBLIC_EXPONENT, (0x01, 0x00, 0x01)),
            (PyKCS11.CKA_ENCRYPT, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_VERIFY, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_VERIFY_RECOVER, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_WRAP, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_LABEL, "My Public Key"),
            (PyKCS11.CKA_ID, keyID),
        ]

        privTemplate = [
            (PyKCS11.CKA_CLASS, PyKCS11.CKO_PRIVATE_KEY),
            (PyKCS11.CKA_TOKEN, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_PRIVATE, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_DECRYPT, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_SIGN, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_SIGN_RECOVER, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_UNWRAP, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_ID, keyID),
        ]

        self.pubKey, self.privKey = self.session.generateKeyPair(
            pubTemplate, privTemplate
        )
        self.assertIsNotNone(self.pubKey)
        self.assertIsNotNone(self.privKey)

        keyID = (0x02,)
        AESKeyTemplate = [
            (PyKCS11.CKA_CLASS, PyKCS11.CKO_SECRET_KEY),
            (PyKCS11.CKA_KEY_TYPE, PyKCS11.CKK_AES),
            (PyKCS11.CKA_TOKEN, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_PRIVATE, PyKCS11.CK_FALSE),
            (PyKCS11.CKA_ENCRYPT, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_DECRYPT, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_SIGN, PyKCS11.CK_FALSE),
            (PyKCS11.CKA_EXTRACTABLE, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_VERIFY, PyKCS11.CK_FALSE),
            (PyKCS11.CKA_VALUE_LEN, 32),
            (PyKCS11.CKA_LABEL, "TestAESKey"),
            (PyKCS11.CKA_ID, keyID),
        ]

        # make the key extractable
        AESKeyTemplate.append((PyKCS11.CKA_EXTRACTABLE, PyKCS11.CK_TRUE))

        self.AESKey = self.session.generateKey(AESKeyTemplate)
        self.assertIsNotNone(self.AESKey)

        # buffer of 32 bytes 0x42
        DataIn = [42] * 32

        mechanism = PyKCS11.Mechanism(PyKCS11.CKM_AES_ECB)
        DataOut = self.session.encrypt(self.AESKey, DataIn, mechanism)
        # print("DataOut", DataOut)

        DataCheck = self.session.decrypt(self.AESKey, DataOut, mechanism)
        # print("DataCheck:", DataCheck)

        # check we can encrypt/decrypt with the AES key
        self.assertSequenceEqual(DataIn, DataCheck)

        # wrap using CKM_RSA_PKCS_OAEP + CKG_MGF1_SHA1
        mechanismWrap = PyKCS11.RSAOAEPMechanism(
            PyKCS11.CKM_SHA_1, PyKCS11.CKG_MGF1_SHA1
        )
        wrapped = self.session.wrapKey(self.pubKey, self.AESKey, mechanismWrap)
        self.assertIsNotNone(wrapped)

        # destroy the original key
        self.session.destroyObject(self.AESKey)

        # unwrap
        template = [
            (PyKCS11.CKA_CLASS, PyKCS11.CKO_SECRET_KEY),
            (PyKCS11.CKA_KEY_TYPE, PyKCS11.CKK_AES),
            (PyKCS11.CKA_TOKEN, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_PRIVATE, PyKCS11.CK_FALSE),
            (PyKCS11.CKA_ENCRYPT, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_DECRYPT, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_SIGN, PyKCS11.CK_FALSE),
            (PyKCS11.CKA_VERIFY, PyKCS11.CK_FALSE),
        ]
        unwrapped = self.session.unwrapKey(
            self.privKey, wrapped, template, mechanismWrap
        )
        self.assertIsNotNone(unwrapped)

        DataCheck = self.session.decrypt(unwrapped, DataOut, mechanism)
        # print("DataCheck:", DataCheck)

        # check we can decrypt with the unwrapped AES key
        self.assertSequenceEqual(DataIn, DataCheck)

        # cleanup
        self.session.destroyObject(unwrapped)

        self.session.destroyObject(self.pubKey)
        self.session.destroyObject(self.privKey)

    def test_wrapKey_UNWRAP_TEMPLATE(self):
        keyID = (0x01,)
        pubTemplate = [
            (PyKCS11.CKA_CLASS, PyKCS11.CKO_PUBLIC_KEY),
            (PyKCS11.CKA_LABEL, "RSA Public Key"),
            (PyKCS11.CKA_KEY_TYPE, PyKCS11.CKK_RSA),
            (PyKCS11.CKA_TOKEN, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_PRIVATE, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_VERIFY, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_MODULUS_BITS, 2048),
            (PyKCS11.CKA_PUBLIC_EXPONENT, (0x01, 0x00, 0x01)),
            (PyKCS11.CKA_ID, keyID),
            (PyKCS11.CKA_WRAP, PyKCS11.CK_TRUE),
        ]

        unwrap_template = [
            (PyKCS11.CKA_EXTRACTABLE, PyKCS11.CK_FALSE),
        ]

        privTemplate = [
            (PyKCS11.CKA_CLASS, PyKCS11.CKO_PRIVATE_KEY),
            (PyKCS11.CKA_LABEL, "RSA Private Key"),
            (PyKCS11.CKA_KEY_TYPE, PyKCS11.CKK_RSA),
            (PyKCS11.CKA_TOKEN, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_PRIVATE, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_SIGN, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_ID, keyID),
            (PyKCS11.CKA_UNWRAP, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_UNWRAP_TEMPLATE, unwrap_template),
        ]

        if self.SoftHSMversion < 2:
            self.skipTest("generateKey() only supported by SoftHSM >= 2")

        self.pubKey, self.privKey = self.session.generateKeyPair(
            pubTemplate, privTemplate
        )
        self.assertIsNotNone(self.pubKey)
        self.assertIsNotNone(self.privKey)

        keyID = (0x02,)
        AESKeyTemplate = [
            (PyKCS11.CKA_CLASS, PyKCS11.CKO_SECRET_KEY),
            (PyKCS11.CKA_KEY_TYPE, PyKCS11.CKK_AES),
            (PyKCS11.CKA_TOKEN, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_PRIVATE, PyKCS11.CK_FALSE),
            (PyKCS11.CKA_ENCRYPT, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_DECRYPT, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_SIGN, PyKCS11.CK_FALSE),
            (PyKCS11.CKA_EXTRACTABLE, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_VERIFY, PyKCS11.CK_FALSE),
            (PyKCS11.CKA_VALUE_LEN, 32),
            (PyKCS11.CKA_LABEL, "TestAESKey"),
            (PyKCS11.CKA_ID, keyID),
        ]

        # make the key extractable
        AESKeyTemplate.append((PyKCS11.CKA_EXTRACTABLE, PyKCS11.CK_TRUE))

        self.AESKey = self.session.generateKey(AESKeyTemplate)
        self.assertIsNotNone(self.AESKey)

        # buffer of 32 bytes 0x42
        DataIn = [42] * 32

        mechanism = PyKCS11.Mechanism(PyKCS11.CKM_AES_ECB)
        DataOut = self.session.encrypt(self.AESKey, DataIn, mechanism)
        # print("DataOut", DataOut)

        DataCheck = self.session.decrypt(self.AESKey, DataOut, mechanism)
        # print("DataCheck:", DataCheck)

        # check we can encrypt/decrypt with the AES key
        self.assertSequenceEqual(DataIn, DataCheck)

        # wrap
        mechanismWrap = PyKCS11.RSAOAEPMechanism(
            PyKCS11.CKM_SHA_1, PyKCS11.CKG_MGF1_SHA1
        )
        wrapped = self.session.wrapKey(self.pubKey, self.AESKey, mechanismWrap)
        self.assertIsNotNone(wrapped)

        # destroy the original key
        self.session.destroyObject(self.AESKey)

        # unwrap
        template = [
            (PyKCS11.CKA_TOKEN, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_CLASS, PyKCS11.CKO_SECRET_KEY),
            (PyKCS11.CKA_KEY_TYPE, PyKCS11.CKK_AES),
            (PyKCS11.CKA_EXTRACTABLE, PyKCS11.CK_FALSE),
        ]
        unwrapped = self.session.unwrapKey(
            self.privKey, wrapped, template, mechanismWrap
        )
        self.assertIsNotNone(unwrapped)

        DataCheck = self.session.decrypt(unwrapped, DataOut, mechanism)
        # print("DataCheck:", DataCheck)

        # check we can decrypt with the unwrapped AES key
        self.assertSequenceEqual(DataIn, DataCheck)

        attributes = self.session.getAttributeValue(
            unwrapped, [PyKCS11.CKA_EXTRACTABLE]
        )
        self.assertSequenceEqual(attributes, [False])

        # cleanup
        self.session.destroyObject(unwrapped)

        self.session.destroyObject(self.pubKey)
        self.session.destroyObject(self.privKey)



#!/usr/bin/env python3

# use:
# ./run_test.py
# ./run_test.py test_ckbytelist.py

import os
import sys
import unittest

import get_PYKCS11LIB

pattern = "test*.py"
if len(sys.argv) > 1:
    pattern = sys.argv[1]

if __name__ == "__main__":
    lib = get_PYKCS11LIB.get_PYKCS11LIB()
    os.environ["PYKCS11LIB"] = lib

    tl = unittest.TestLoader()
    suite = tl.discover("test", pattern=pattern)
    result = unittest.TextTestRunner(verbosity=2).run(suite)
    if result.errors or result.failures:
        sys.exit(1)



#!/usr/bin/env python3

"""
#   contributed by Alex Railean, a dot railean at dekart.com
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA.
"""

import PyKCS11

# pylint: disable=duplicate-code

pkcs11 = PyKCS11.PyKCS11Lib()
pkcs11.load()  # define environment variable PYKCS11LIB=YourPKCS11Lib

slot = pkcs11.getSlotList(tokenPresent=True)[0]
session = pkcs11.openSession(slot, PyKCS11.CKF_RW_SESSION)

pin = "1234"
session.login(pin, PyKCS11.CKU_USER)


# ############   key-pair generation    ##########################
# The first step in the process is to create the key-templates. See PKCS#11
# `10.8 Public key objects` to learn which attributes are available. Section
# 10.9 covers private keys.
label = "pkcs_is_fun"  # just a label for identifying objects
key_length = 2048  # key-length in bits

# the key_id has to be the same for both objects, it will also be necessary
# when importing the certificate, to ensure it is linked with these keys.
key_id = (0x01,)

public_template = [
    (PyKCS11.CKA_CLASS, PyKCS11.CKO_PUBLIC_KEY),
    (PyKCS11.CKA_PRIVATE, PyKCS11.CK_FALSE),
    (PyKCS11.CKA_TOKEN, PyKCS11.CK_TRUE),
    (PyKCS11.CKA_ENCRYPT, PyKCS11.CK_TRUE),
    (PyKCS11.CKA_VERIFY, PyKCS11.CK_TRUE),
    (PyKCS11.CKA_WRAP, PyKCS11.CK_TRUE),
    (PyKCS11.CKA_KEY_TYPE, PyKCS11.CKK_RSA),
    (PyKCS11.CKA_VERIFY_RECOVER, PyKCS11.CK_TRUE),
    (PyKCS11.CKA_LABEL, label),
    (PyKCS11.CKA_MODULUS_BITS, key_length),
    (PyKCS11.CKA_ID, key_id),
]


private_template = [
    (PyKCS11.CKA_CLASS, PyKCS11.CKO_PRIVATE_KEY),
    (PyKCS11.CKA_PRIVATE, PyKCS11.CK_TRUE),
    (PyKCS11.CKA_TOKEN, PyKCS11.CK_TRUE),
    (PyKCS11.CKA_SENSITIVE, PyKCS11.CK_TRUE),
    (PyKCS11.CKA_DECRYPT, PyKCS11.CK_TRUE),
    (PyKCS11.CKA_SIGN, PyKCS11.CK_TRUE),
    (PyKCS11.CKA_UNWRAP, PyKCS11.CK_TRUE),
    (PyKCS11.CKA_LABEL, label),
    (PyKCS11.CKA_ID, key_id),
]

session.generateKeyPair(public_template, private_template)
# ############# the keys were generated and stored on the card ###############


# At this point your keys are in the card, the private key is marked as
# non-exportable.
# To add a certificate, you have to prepare a CSR in PKCS#10 format and sign
# it with the key.
# It can be done with OpenSSL by typing the commands below in a terminal
# 1. openssl
# 2. engine dynamic -pre SO_PATH:/usr/lib/engines/engine_pkcs11.so -pre
#    ID:pkcs11 -pre LIST_ADD:1 -pre LOAD -pre MODULE_PATH:libacospkcs11.so
# 3. req -engine pkcs11 -keyform engine -key 0:1 -new -text -out
#    /tmp/newcert.csr -subj "/O=users/CN=Test User"
# Note that `-key 0:1` means "use slot 0, with key id 1"
# The CSR will be printed to stdout and saved to /tmp/newcert.csr


# The CSR is sent to a CA, which will issue a certificate and send it back.
# Once you have the certificate, it has to be converted to BER (if it isn't
# already). It will be assumed that the raw data of the certificate are in
# `cert`.

cert = b"replace this with the raw certificate itself"
subject = b"foobar"


# keep in mind that certain elements, such as the subject, must not be
# passed as a string, but as an ASN1-encoded structure. Consider using
# pyasn1 to do that, see http://pyasn1.sourceforge.net/
cert_template = [
    (PyKCS11.CKA_CLASS, PyKCS11.CKO_CERTIFICATE),
    (PyKCS11.CKA_PRIVATE, PyKCS11.CK_FALSE),
    (PyKCS11.CKA_LABEL, label),
    (PyKCS11.CKA_TOKEN, PyKCS11.CK_TRUE),
    (PyKCS11.CKA_CERTIFICATE_TYPE, PyKCS11.CKC_X_509),
    (PyKCS11.CKA_MODIFIABLE, PyKCS11.CK_TRUE),
    (PyKCS11.CKA_VALUE, cert),  # must be BER-encoded
    (
        PyKCS11.CKA_SUBJECT,
        subject,
    ),  # must be set and DER, see Table 24, X.509 Certificate Object Attributes
    (
        PyKCS11.CKA_ID,
        key_id,
    ),  # must be set, and DER see Table 24, X.509 Certificate Object Attributes
]

# create the certificate object
session.createObject(cert_template)

# logout
session.logout()
session.closeSession()


# At this point the certificate is on the card too. Some GUI tools
# might display it as invalid, in that case make sure that the
# entire certificate chain is available in the certificate store.



# pylint: disable=missing-module-docstring
# pylint: disable=missing-class-docstring
# pylint: disable=missing-function-docstring

import unittest

from PyKCS11 import PyKCS11


class TestUtil(unittest.TestCase):
    def setUp(self):
        self.pkcs11 = PyKCS11.PyKCS11Lib()
        self.pkcs11.load()

        # get SoftHSM major version
        self.SoftHSMversion = self.pkcs11.getInfo().libraryVersion

        self.slot = self.pkcs11.getSlotList(tokenPresent=True)[0]
        self.session = self.pkcs11.openSession(
            self.slot, PyKCS11.CKF_SERIAL_SESSION | PyKCS11.CKF_RW_SESSION
        )
        self.session.login("1234")

        self.aesKeyTemplate = [
            (PyKCS11.CKA_CLASS, PyKCS11.CKO_SECRET_KEY),
            (PyKCS11.CKA_KEY_TYPE, PyKCS11.CKK_AES),
            (PyKCS11.CKA_TOKEN, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_PRIVATE, PyKCS11.CK_FALSE),
            (PyKCS11.CKA_ENCRYPT, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_DECRYPT, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_SIGN, PyKCS11.CK_FALSE),
            (PyKCS11.CKA_VERIFY, PyKCS11.CK_FALSE),
            (PyKCS11.CKA_VALUE_LEN, 32),
            (PyKCS11.CKA_LABEL, "TestAESKey"),
        ]

        self.aesBlockSize = 16

    def tearDown(self):
        self.session.logout()
        self.pkcs11.closeAllSessions(self.slot)
        del self.pkcs11

    def test_symetric(self):
        # AES CBC with IV
        mechanism = PyKCS11.Mechanism(PyKCS11.CKM_AES_CBC, "1234567812345678")
        self.assertIsNotNone(mechanism)

        if self.SoftHSMversion < (2, 0):
            self.skipTest("generateKey() only supported by SoftHSM >= 2.0")

        keyID = (0x01,)
        AESKeyTemplate = self.aesKeyTemplate + [(PyKCS11.CKA_ID, keyID)]

        AESKey = self.session.generateKey(AESKeyTemplate)
        self.assertIsNotNone(AESKey)

        # buffer of 32 bytes 0x00
        DataIn = [0] * 32
        # print("DataIn:", DataIn)

        # AES CBC with IV
        mechanism = PyKCS11.Mechanism(PyKCS11.CKM_AES_CBC, "1234567812345678")

        # find the first secret key
        symKey = self.session.findObjects(
            [(PyKCS11.CKA_CLASS, PyKCS11.CKO_SECRET_KEY), (PyKCS11.CKA_ID, keyID)]
        )[0]

        DataOut = self.session.encrypt(symKey, DataIn, mechanism)
        # print("DataOut", DataOut)

        DataCheck = self.session.decrypt(symKey, DataOut, mechanism)
        # print("DataCheck:", DataCheck)

        self.assertSequenceEqual(DataIn, DataCheck)

        # AES ECB with previous IV as Data
        mechanism = PyKCS11.Mechanism(PyKCS11.CKM_AES_ECB)

        # same as '1234567812345678' (the IV) but as a list
        DataECBIn = [49, 50, 51, 52, 53, 54, 55, 56, 49, 50, 51, 52, 53, 54, 55, 56]
        # print("DataECBIn:", DataECBIn)
        DataECBOut = self.session.encrypt(symKey, DataECBIn, mechanism)
        # print("DataECBOut:", DataECBOut)

        DataECBCheck = self.session.decrypt(symKey, DataECBOut, mechanism)
        # print("DataECBCheck:", DataECBCheck)

        self.assertSequenceEqual(DataECBIn, DataECBCheck)

        # check the AES CBC computation is the same as the AES ECB
        # 1st block
        self.assertSequenceEqual(DataOut[:16], DataECBOut)

        # since the input is full of 0 we just pass the previous output
        DataECBOut2 = self.session.encrypt(symKey, DataECBOut, mechanism)
        # print("DataECBOut2", DataECBOut2)

        # 2nd block
        self.assertSequenceEqual(DataOut[16:], DataECBOut2)

        # AES CTR with IV
        mechanism = PyKCS11.AES_CTR_Mechanism(128, "1234567812345678")

        DataOut = self.session.encrypt(symKey, DataIn, mechanism)
        # print("DataOut", DataOut)

        DataCheck = self.session.decrypt(symKey, DataOut, mechanism)
        # print("DataCheck:", DataCheck)

        self.assertSequenceEqual(DataIn, DataCheck)

        #
        # test CK_GCM_PARAMS
        #

        if self.SoftHSMversion <= (2, 2):
            self.skipTest("CKM_AES_GCM only supported by SoftHSM > 2.2")

        AES_GCM_IV_SIZE = 12
        AES_GCM_TAG_SIZE = 16
        iv = [42] * AES_GCM_IV_SIZE
        aad = "plaintext aad"
        tagBits = AES_GCM_TAG_SIZE * 8
        mechanism = PyKCS11.AES_GCM_Mechanism(iv, aad, tagBits)

        DataOut = self.session.encrypt(symKey, DataIn, mechanism)
        # print("DataOut", DataOut)

        DataCheck = self.session.decrypt(symKey, DataOut, mechanism)
        # print("DataCheck:", DataCheck)

        self.assertSequenceEqual(DataIn, DataCheck)

        self.session.destroyObject(AESKey)

    def test_multi_part_symmetric(self):
        iv = self.session.generateRandom(self.aesBlockSize)

        aesMechanisms = {
            "CKM_AES_ECB": PyKCS11.Mechanism(PyKCS11.CKM_AES_ECB),
            "CKM_AES_CBC": PyKCS11.Mechanism(PyKCS11.CKM_AES_CBC, iv),
            "CKM_AES_CTR": PyKCS11.AES_CTR_Mechanism(128, iv),
        }

        keyID = (0x02,)
        aesKey = self.session.generateKey(
            self.aesKeyTemplate + [(PyKCS11.CKA_ID, keyID)]
        )
        self.assertIsNotNone(aesKey)

        for mechName in aesMechanisms:
            with self.subTest(mechName=mechName):
                if not mechName in self.pkcs11.getMechanismList(self.slot):
                    self.skipTest(f"{mechName} is not supported by the token")
                self._run_multipart_encrypt_decrypt(aesMechanisms[mechName], aesKey)

        self.session.destroyObject(aesKey)

    def _run_multipart_encrypt_decrypt(self, mechanism, key):
        self.assertIsNotNone(mechanism)

        self.session.encryptInit(mechanism, key)

        data = self.session.generateRandom(self.aesBlockSize * 8)

        encData = []
        dataPos = 0
        # encrypt data using parts of different length,
        # but each of them must be a multiple of the block size
        for m in [1, 2, 3, 4]:  # total 8 blocks
            dataPart = list(data[dataPos : dataPos + m * self.aesBlockSize])
            encData += self.session.encryptUpdate(dataPart)
            dataPos += m * self.aesBlockSize
        encData += self.session.encryptFinal()

        self.session.decryptInit(mechanism, key)

        decData = []
        dataPos = 0
        for m in [3, 1, 2, 4]:  # total 8 blocks
            encPart = list(encData[dataPos : dataPos + m * self.aesBlockSize])
            decData += self.session.decryptUpdate(encPart)
            dataPos += m * self.aesBlockSize
        decData += self.session.decryptFinal()

        self.assertSequenceEqual(data, decData)

        # initiate another encryption to check that
        # the previous operation has been terminated
        self.session.encryptInit(mechanism, key)
        self.session.encryptFinal()

    def test_multi_part_aead(self):
        if "CKM_AES_GCM" not in self.pkcs11.getMechanismList(self.slot):
            self.skipTest("CKM_AES_GCM is not supported by the token")

        keyID = (0x03,)
        aesKey = self.session.generateKey(
            self.aesKeyTemplate + [(PyKCS11.CKA_ID, keyID)]
        )
        self.assertIsNotNone(aesKey)

        AES_GCM_IV_SIZE = 12
        AES_GCM_TAG_SIZE = 16
        iv = self.session.generateRandom(AES_GCM_IV_SIZE)
        aad = "plaintext aad"
        tagBits = AES_GCM_TAG_SIZE * 8
        mechanism = PyKCS11.AES_GCM_Mechanism(iv, aad, tagBits)

        self.session.encryptInit(mechanism, aesKey)

        data = self.session.generateRandom()

        encData = self.session.encryptUpdate(data)
        tag = self.session.encryptFinal()
        self.assertTrue(tag)

        self.session.decryptInit(mechanism, aesKey)

        decData = self.session.decryptUpdate(encData)

        # check: since CKM_AES_GCM is an AEAD cipher, no data should be returned until decryptFinal()
        # see https://docs.oasis-open.org/pkcs11/pkcs11-curr/v3.0/os/pkcs11-curr-v3.0-os.html#_Toc30061258
        self.assertFalse(decData)

        decData += self.session.decryptUpdate(tag)
        self.assertFalse(decData)
        decData += self.session.decryptFinal()

        self.assertSequenceEqual(data, list(decData))
        self.session.destroyObject(aesKey)



# pylint: disable=missing-module-docstring
# pylint: disable=missing-class-docstring
# pylint: disable=missing-function-docstring

import unittest

from PyKCS11 import PyKCS11


class TestUtil(unittest.TestCase):
    def setUp(self):
        self.pkcs11 = PyKCS11.PyKCS11Lib()
        self.pkcs11.load()

        # get SoftHSM major version
        self.SoftHSMversion = self.pkcs11.getInfo().libraryVersion[0]

        self.slot = self.pkcs11.getSlotList(tokenPresent=True)[0]
        self.session = self.pkcs11.openSession(
            self.slot, PyKCS11.CKF_SERIAL_SESSION | PyKCS11.CKF_RW_SESSION
        )
        self.session.login("1234")

    def tearDown(self):
        self.session.logout()
        self.pkcs11.closeAllSessions(self.slot)
        self.pkcs11.unload()
        del self.pkcs11

    def test_gost(self):
        if self.SoftHSMversion < 2:
            self.skipTest("generateKeyPair() only supported by SoftHSM >= 2")

        # values from SoftHSMv2
        param_a = (0x06, 0x07, 0x2A, 0x85, 0x03, 0x02, 0x02, 0x23, 0x01)
        param_b = (0x06, 0x07, 0x2A, 0x85, 0x03, 0x02, 0x02, 0x1E, 0x01)

        keyID = (0x23,)
        pubTemplate = [
            (PyKCS11.CKA_TOKEN, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_PRIVATE, PyKCS11.CK_FALSE),
            (PyKCS11.CKA_ENCRYPT, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_VERIFY, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_WRAP, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_LABEL, "My Public Key"),
            (PyKCS11.CKA_ID, keyID),
            (PyKCS11.CKA_GOSTR3410_PARAMS, param_a),
            (PyKCS11.CKA_GOSTR3411_PARAMS, param_b),
        ]

        privTemplate = [
            (PyKCS11.CKA_TOKEN, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_PRIVATE, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_DECRYPT, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_SIGN, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_UNWRAP, PyKCS11.CK_TRUE),
            (PyKCS11.CKA_ID, keyID),
        ]

        # test generate gost key pair
        gen_mechanism = PyKCS11.Mechanism(PyKCS11.CKM_GOSTR3410_KEY_PAIR_GEN, None)
        try:
            (self.pubKey, self.privKey) = self.session.generateKeyPair(
                pubTemplate, privTemplate, gen_mechanism
            )
        except PyKCS11.PyKCS11Error as e:
            if e.value == PyKCS11.CKR_MECHANISM_INVALID:
                self.skipTest("GOST not supported by SoftHSMv2 on Windows?")
            else:
                raise

        self.assertIsNotNone(self.pubKey)
        self.assertIsNotNone(self.privKey)

        # test sign GOSTR3410_WITH_GOSTR3411
        toSign = "Hello world"
        mecha = PyKCS11.Mechanism(PyKCS11.CKM_GOSTR3410_WITH_GOSTR3411, None)

        # sign/verify
        signature = self.session.sign(self.privKey, toSign, mecha)

        result = self.session.verify(self.pubKey, toSign, signature, mecha)
        self.assertTrue(result)

        # test CK_OBJECT_HANDLE.__repr__()
        text = str(self.pubKey)
        self.assertIsNotNone(text)

        self.session.destroyObject(self.pubKey)
        self.session.destroyObject(self.privKey)



# pylint: disable=missing-module-docstring
# pylint: disable=missing-class-docstring
# pylint: disable=missing-function-docstring

import unittest

from PyKCS11 import PyKCS11


class TestUtil(unittest.TestCase):
    def setUp(self):
        self.pkcs11 = PyKCS11.PyKCS11Lib()
        self.pkcs11.load()
        self.slot = self.pkcs11.getSlotList(tokenPresent=True)[0]
        self.session = self.pkcs11.openSession(
            self.slot, PyKCS11.CKF_SERIAL_SESSION | PyKCS11.CKF_RW_SESSION
        )

    def tearDown(self):
        self.pkcs11.closeAllSessions(self.slot)
        del self.pkcs11

    def test_initPin(self):
        # use admin pin
        self.session.login("123456", user_type=PyKCS11.CKU_SO)
        # change PIN
        self.session.initPin("4321")
        self.session.logout()

        # check new PIN
        self.session.login("4321")
        self.session.logout()

        # reset to old PIN
        self.session.login("123456", user_type=PyKCS11.CKU_SO)
        self.session.initPin("1234")
        self.session.logout()

        # check old PIN
        self.session.login("1234")
        self.session.logout()

    def test_setPin(self):
        self.session.login("1234")

        # change PIN
        self.session.setPin("1234", "4321")
        self.session.logout()

        # test new PIN
        self.session.login("4321")

        # revert to old PIN
        self.session.setPin("4321", "1234")
        self.session.logout()

    def test_initToken(self):
        self.pkcs11.closeAllSessions(self.slot)

        # use admin PIN
        self.pkcs11.initToken(self.slot, "123456", "my label")
        self.session = self.pkcs11.openSession(
            self.slot, PyKCS11.CKF_SERIAL_SESSION | PyKCS11.CKF_RW_SESSION
        )
        self.session.login("123456", user_type=PyKCS11.CKU_SO)
        # set user PIN
        self.session.initPin("1234")
        self.session.logout()

    def test_initToken_utf8(self):
        self.pkcs11.closeAllSessions(self.slot)

        # Create a label using UTF-8
        label = "abcéàç"
        # padding with spaces up to 32 _bytes_ (not characters)
        label += " " * (32 - len(label.encode("utf-8")))

        # use admin PIN
        self.pkcs11.initToken(self.slot, "123456", label)
        self.session = self.pkcs11.openSession(
            self.slot, PyKCS11.CKF_SERIAL_SESSION | PyKCS11.CKF_RW_SESSION
        )
        self.session.login("123456", user_type=PyKCS11.CKU_SO)
        # set user PIN
        self.session.initPin("1234")

        token_info = self.pkcs11.getTokenInfo(self.slot)
        self.assertEqual(token_info.label, label)

        self.session.logout()



# pylint: disable=missing-module-docstring
# pylint: disable=missing-class-docstring
# pylint: disable=missing-function-docstring

import unittest

import PyKCS11


class Testutil(unittest.TestCase):
    def test_empty(self):
        ck = PyKCS11.ckbytelist()
        self.assertSequenceEqual(ck, [])

    def test_resize(self):
        ck = PyKCS11.ckbytelist()
        ck.resize(5)
        self.assertSequenceEqual(ck, [0, 0, 0, 0, 0])

    def test_data(self):
        ck = PyKCS11.ckbytelist([0] * 5)
        for index in range(5):
            ck[index] = index
        self.assertSequenceEqual(ck, [0, 1, 2, 3, 4])

    def test_append(self):
        ck = PyKCS11.ckbytelist()
        for index in range(5):
            ck.append(index)
        self.assertSequenceEqual(ck, [0, 1, 2, 3, 4])

    def test_length0(self):
        ck = PyKCS11.ckbytelist()
        self.assertEqual(len(ck), 0)

    def test_length5(self):
        ck = PyKCS11.ckbytelist([0] * 5)
        self.assertEqual(len(ck), 5)

    def test_string(self):
        ck = PyKCS11.ckbytelist()
        ck.resize(5)
        for index in range(5):
            ck[index] = index
        self.assertEqual(str(ck), "[0, 1, 2, 3, 4]")

    def test_init_list0(self):
        ck = PyKCS11.ckbytelist([])
        self.assertSequenceEqual(ck, [])

    def test_init_list1(self):
        ck = PyKCS11.ckbytelist([1])
        self.assertSequenceEqual(ck, [1])

    def test_init_list5(self):
        ck = PyKCS11.ckbytelist([0, 1, 2, 3, 4])
        self.assertSequenceEqual(ck, [0, 1, 2, 3, 4])

    def test_init_str(self):
        ck = PyKCS11.ckbytelist("ABC")
        self.assertSequenceEqual(ck, [65, 66, 67])

    def test_init_bytes(self):
        ck = PyKCS11.ckbytelist(b"ABC")
        self.assertSequenceEqual(ck, [65, 66, 67])

    def test_init_ckbytelist(self):
        ck1 = PyKCS11.ckbytelist(b"ABC")
        ck2 = PyKCS11.ckbytelist(ck1)
        self.assertSequenceEqual(ck2, [65, 66, 67])

    def test_unknown_format(self):
        with self.assertRaises(PyKCS11.PyKCS11Error) as cm:
            PyKCS11.ckbytelist({})

        the_exception = cm.exception
        self.assertEqual(the_exception.value, -3)

        # Python 3 and later
        type_str = "<class 'dict'>"

        self.assertEqual(the_exception.text, type_str)
        self.assertEqual(str(the_exception), f"Unknown format ({type_str})")

    def test_concat(self):
        a = PyKCS11.ckbytelist("ABC")
        b = PyKCS11.ckbytelist("DEF")
        c = a + b
        self.assertSequenceEqual(c, [65, 66, 67] + [68, 69, 70])

    def test_add(self):
        a = PyKCS11.ckbytelist("ABC")
        b = PyKCS11.ckbytelist("DEF")
        a += b
        self.assertSequenceEqual(a, [65, 66, 67] + [68, 69, 70])

    def test_lowlevel(self):
        ck = PyKCS11.LowLevel.ckbytelist()
        self.assertSequenceEqual(ck, [])

        ck = PyKCS11.LowLevel.ckbytelist([0, 1, 2, 3, 4])
        self.assertSequenceEqual(ck, [0, 1, 2, 3, 4])

        with self.assertRaises(TypeError):
            ck = PyKCS11.LowLevel.ckbytelist("ABC")

        ck = PyKCS11.LowLevel.ckbytelist(b"ABC")
        self.assertSequenceEqual(ck, [65, 66, 67])


if __name__ == "__main__":
    unittest.main()



#!/usr/bin/env python3

"""
#   Copyright (C) 2006-2014 Ludovic Rousseau (ludovic.rousseau@free.fr)
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA.
"""

import getopt
import platform
import sys

import PyKCS11

# from http://aspn.activestate.com/ASPN/Cookbook/Python/Recipe/142812
# Title: Hex dumper
# Submitter: Sebastien Keim (other recipes)
# Last Updated: 2002/08/05
# Version no: 1.0


def dump(src, length=16):
    """dump a binary buffer"""

    def to_ascii(x):
        if 32 <= x <= 127:
            return chr(x)
        return "."

    N = 0
    result = ""
    while src:
        h, src = src[:length], src[length:]
        text_hexa = " ".join([f"{x:02X}" for x in h])
        text_ascii = "".join(map(to_ascii, h))
        result += f"{N:04X}   {text_hexa:{length * 3}}   {text_ascii}\n"
        N += length
    return result


def usage():
    """usage"""
    print("Usage:", sys.argv[0], end=" ")
    print("[-a][--all]", end=" ")
    print("[-p pin][--pin=pin] (use --pin=NULL for pinpad)", end=" ")
    print("[-c lib][--lib=lib]", end=" ")
    print("[-s slot][--slot=slot]", end=" ")
    print("[-h][--help]", end=" ")
    print()


try:
    opts, args = getopt.getopt(
        sys.argv[1:],
        "p:c:Sdhs:a",
        ["pin=", "lib=", "slot=", "help", "all"],
    )
except getopt.GetoptError:
    # print help information and exit:
    usage()
    sys.exit(2)

pin_available = False
lib = None
slot = None
token_present = True
for o, a in opts:
    if o in ("-h", "--help"):
        usage()
        sys.exit()
    elif o in ("-p", "--pin"):
        pin = a
        if pin == "NULL":
            pin = None
        pin_available = True
    elif o in ("-c", "--lib"):
        lib = a
        print("using PKCS11 lib:", lib)
    elif o in ("-s", "--slot"):
        slot = int(a)
    if o in ("-a", "--all"):
        token_present = False

red = blue = magenta = normal = ""
if sys.stdout.isatty() and platform.system().lower() != "windows":
    red = "\x1b[01;31m"
    blue = "\x1b[34m"
    magenta = "\x1b[35m"
    normal = "\x1b[0m"

format_long = magenta + "  %s:" + blue + " %s (%s)" + normal
format_binary = magenta + "  %s:" + blue + " %d bytes" + normal
format_normal = magenta + "  %s:" + blue + " %s" + normal

pkcs11 = PyKCS11.PyKCS11Lib()
pkcs11.load(lib)
info = pkcs11.getInfo()
print("Library manufacturerID:", info.manufacturerID)

slots = pkcs11.getSlotList(token_present)
print("Available Slots:", len(slots), slots)

if slot is not None:
    slots = [slots[slot]]
    print("Using slot:", slots[0])

for s in slots:
    try:
        i = pkcs11.getSlotInfo(s)
        print("Slot no:", s)
        print(format_normal % ("slotDescription", i.slotDescription.strip()))
        print(format_normal % ("manufacturerID ", i.manufacturerID.strip()))
        print(format_normal % ("hardwareVersion", i.hardwareVersion))
        print(format_normal % ("firmwareVersion", i.firmwareVersion))
        print(format_normal % ("flags          ", ", ".join(i.flags2text())))

        if not i.flags & PyKCS11.CKF_TOKEN_PRESENT:
            print("  Token not present")
            continue

        t = pkcs11.getTokenInfo(s)
        print("TokenInfo")
        print(format_normal % ("label", t.label.strip()))
        print(format_normal % ("manufacturerID", t.manufacturerID.strip()))
        print(format_normal % ("model", t.model.strip()))

        session = pkcs11.openSession(s)
        print(f"Opened session 0x{session.session.value():08X}")
        if pin_available:
            try:
                if (pin is None) and (
                    PyKCS11.CKF_PROTECTED_AUTHENTICATION_PATH & t.flags
                ):
                    print(f"\nEnter your PIN for {t.label.strip()} on the pinpad")
                session.login(pin=pin)
            except PyKCS11.PyKCS11Error as e:
                print("login failed, exception:", e)
                break

        objects = session.findObjects()
        print()
        print(f"Found {len(objects)} objects: {[x.value() for x in objects]}")

        all_attributes = list(PyKCS11.CKA.keys())
        # remove the CKR_ATTRIBUTE_SENSITIVE attributes since we can't get
        # their values and will get an exception instead
        all_attributes.remove(PyKCS11.CKA_PRIVATE_EXPONENT)
        all_attributes.remove(PyKCS11.CKA_PRIME_1)
        all_attributes.remove(PyKCS11.CKA_PRIME_2)
        all_attributes.remove(PyKCS11.CKA_EXPONENT_1)
        all_attributes.remove(PyKCS11.CKA_EXPONENT_2)
        all_attributes.remove(PyKCS11.CKA_COEFFICIENT)
        # only use the integer values and not the strings like 'CKM_RSA_PKCS'
        all_attributes = [e for e in all_attributes if isinstance(e, int)]

        n_obj = 1
        for o in objects:
            print()
            print(
                (
                    red
                    + "==================== Object: %d/%d (%d) ===================="
                    + normal
                )
                % (n_obj, len(objects), o.value())
            )
            n_obj += 1
            try:
                attributes = session.getAttributeValue(o, all_attributes)
            except PyKCS11.PyKCS11Error as e:
                print("getAttributeValue failed:", e)
                continue
            attrDict = dict(list(zip(all_attributes, attributes)))

            print("Dumping attributes:")
            for q, a in zip(all_attributes, attributes):
                if a is None:
                    # undefined (CKR_ATTRIBUTE_TYPE_INVALID) attribute
                    continue
                if q == PyKCS11.CKA_CLASS:
                    print(format_long % (PyKCS11.CKA[q], PyKCS11.CKO[a], a))
                elif q == PyKCS11.CKA_CERTIFICATE_TYPE:
                    print(format_long % (PyKCS11.CKA[q], PyKCS11.CKC[a], a))
                elif q == PyKCS11.CKA_KEY_TYPE:
                    print(format_long % (PyKCS11.CKA[q], PyKCS11.CKK[a], a))
                elif session.isBin(q):
                    print(format_binary % (PyKCS11.CKA[q], len(a)))
                    if a:
                        print(dump(a), end="")
                elif q == PyKCS11.CKA_SERIAL_NUMBER:
                    print(format_binary % (PyKCS11.CKA[q], len(a)))
                    if a:
                        print(dump(a), end="")
                else:
                    print(format_normal % (PyKCS11.CKA[q], a))
        print()

        if pin_available:
            try:
                session.logout()
            except PyKCS11.PyKCS11Error as e:
                print("logout failed, exception:", e)
                break

        session.closeSession()

    except PyKCS11.PyKCS11Error as e:
        print("Error:", e)
        raise



import platform
import shutil
from os import path
from shutil import which
from sys import version_info as pyver

from setuptools import Extension, setup
from setuptools.command.build_py import build_py

description = """A complete PKCS#11 wrapper for Python.
You can use any PKCS#11 (aka CryptoKi) module such as the PSM which
comes as part of mozilla or the various modules supplied by vendors of
hardware crypto tokens, and almost all PKCS#11 functions and data types.
The wrapper has been generated with the help of the SWIG compiler."""

classifiers = [
    "Development Status :: 5 - Production/Stable",
    "Intended Audience :: Developers",
    "License :: OSI Approved :: GNU General Public License (GPL)",
    "Natural Language :: English",
    "Operating System :: Microsoft :: Windows",
    "Operating System :: OS Independent",
    "Operating System :: Unix",
    "Programming Language :: C",
    "Programming Language :: C++",
    "Programming Language :: Python",
    "Topic :: Security :: Cryptography",
    "Topic :: Software Development :: Libraries :: Python Modules",
]

lib_dirs = []
inc_dirs = ["src"]
# some OS, such as FreeBSD, uses /usr/local folders
if path.exists("/usr/local"):
    lib_dirs.append("/usr/local/lib")
    inc_dirs.append("/usr/local/include")
source_files = [
    "src/ck_attribute_smart.cpp",
    "src/pkcs11lib.cpp",
    "src/pykcs11string.cpp",
    "src/utility.cpp",
    "src/pykcs11.i",
    "src/pykcs11.cpp",
]
define_macros = []
extra_compile_args = []
extra_link_args = []
if platform.system().lower() == "windows":
    source_files.append("src/dyn_win32.c")
    libraries_val = ["python%d%d" % pyver[:2]]
    extra_compile_args = ["/Fdvc70.pdb", "/Zi", "/GR", "/EHsc", "/std:c++20"]
    extra_link_args = [
        "/DEBUG",
        "/PDB:_LowLevel.pdb",
        "/SUBSYSTEM:WINDOWS",
        "/OPT:REF",
        "/OPT:ICF",
    ]
else:
    source_files.append("src/dyn_unix.c")
    libraries_val = []


class MyBuild(build_py):
    def run(self):
        if which("swig") is None:
            print("Install swig and try again")
            print("")
            exit(1)
        self.run_command("build_ext")
        shutil.copy("src/LowLevel.py", "PyKCS11")
        build_py.run(self)


setup(
    name="PyKCS11",
    version="1.5.18",
    description="A Full PKCS#11 wrapper for Python",
    keywords="crypto,pki,pkcs11,c++",
    classifiers=classifiers,
    platforms="Win32 Unix",
    long_description=description,
    author="Giuseppe Amato (Midori)",
    author_email="paipai@tiscali.it",
    maintainer="Ludovic Rousseau",
    maintainer_email="ludovic.rousseau@free.fr",
    url="https://github.com/LudovicRousseau/PyKCS11",
    download_url="http://sourceforge.net/projects/pkcs11wrap/files/pykcs11/",
    license="GPL",
    cmdclass={"build_py": MyBuild},
    ext_modules=[
        Extension(
            name="PyKCS11._LowLevel",
            sources=source_files,
            include_dirs=inc_dirs,
            library_dirs=lib_dirs,
            libraries=libraries_val,
            define_macros=define_macros,
            swig_opts=["-c++"],
            extra_compile_args=extra_compile_args,
            extra_link_args=extra_link_args,
        )
    ],
    py_modules=["PyKCS11.__init__", "PyKCS11.LowLevel", "PyKCS11.constants"],
)
